# Generated by Django 2.2.4 on 2019-10-16 11:06

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('schema', '0011_auto_20191016_1035'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='projector',
            name='manufacturer',
        ),
        migrations.RemoveField(
            model_name='projector',
            name='mount',
        ),
        migrations.RemoveField(
            model_name='projector',
            name='negative_size',
        ),
        migrations.RemoveField(
            model_name='projector',
            name='owner',
        ),
        migrations.AlterField(
            model_name='accessory',
            name='type',
            field=models.CharField(choices=[('Battery grip', 'Battery grip'), ('Case', 'Case'), ('Film back', 'Film back'), ('Focusing screen', 'Focusing screen'), ('Lens hood', 'Lens hood'), ('Lens cap', 'Lens cap'), ('Power winder', 'Power winder'), ('Viewfinder', 'Viewfinder'), ('Rangefinder', 'Rangefinder'), ('Projector', 'Projector'), ('Light meter', 'Light meter')], help_text='Type of accessory', max_length=15),
        ),
        migrations.DeleteModel(
            name='LightMeter',
        ),
        migrations.DeleteModel(
            name='Projector',
        ),
    ]



from django.db import models
from djmoney.models.fields import MoneyField
from django.core.validators import MinValueValidator, MaxValueValidator
from django.core.exceptions import ValidationError
from djchoices import DjangoChoices, ChoiceItem
from datetime import datetime
from math import sqrt
from django_currentuser.db.models import CurrentUserField
import re

# Create your models here.
class Manufacturer(models.Model):
  name = models.CharField(help_text='Name of the manufacturer', max_length=45, blank=True, unique=True)
  city = models.CharField(help_text='City in which the manufacturer is based', max_length=45, blank=True, null=True)
  country = models.CharField(help_text='Country in which the manufacturer is based', max_length=45, blank=True, null=True)
  url = models.URLField(verbose_name='URL', help_text='URL to the manufacturers main website', max_length=45, blank=True, null=True)
  founded = models.PositiveIntegerField(help_text='Year in which the manufacturer was founded', blank=True, null=True)
  dissolved = models.PositiveIntegerField(help_text='Year in which the manufacturer was dissolved', blank=True, null=True)
  def __str__(self):
    return self.name
  class Meta:
    ordering = ['name']
    verbose_name_plural = "manufacturers"
  def clean(self):
    # City/country
    if self.country is None and self.city is not None:
      raise ValidationError({
        'country': ValidationError(('Must specify country if city is given')),
      })
    # Founded/dissolved
    if self.founded is not None and self.dissolved is not None and self.founded > self.dissolved:
      raise ValidationError({
        'founded': ValidationError(('Founded date must be earlier than dissolved date')),
        'dissolved': ValidationError(('Dissolved date must be later than founded date')),
      })
    if self.founded is not None and self.founded > datetime.now().year:
      raise ValidationError({
        'founded': ValidationError(('Founded date must be in the past')),
      })
    if self.dissolved is not None and self.dissolved > datetime.now().year:
      raise ValidationError({
        'dissolved': ValidationError(('Dissolved date must be in the past')),
      })

# Table to list all archives that exist for storing physical media
class Archive(models.Model):
  # Choices for archive types
  class ArchiveType(DjangoChoices):
    Negative = ChoiceItem()
    Slide = ChoiceItem()
    Print = ChoiceItem()

  # Choices for archive storage
  class ArchiveStorage(DjangoChoices):
    Ringbinder = ChoiceItem()
    Folder = ChoiceItem()
    Box = ChoiceItem()
    Portfolio = ChoiceItem()
    Slide_tray = ChoiceItem()

  type = models.CharField(max_length=8, choices=ArchiveType.choices, help_text='What is stored in this archive?')
  name = models.CharField(help_text='Name of this archive', max_length=45, unique=True)
  max_width = models.PositiveIntegerField(help_text='Maximum width of media that this archive can store', blank=True, null=True)
  max_height = models.PositiveIntegerField(help_text='Maximum height of media that this archive can store', blank=True, null=True)
  location = models.CharField(help_text='Location of this archive', max_length=45, blank=True, null=True)
  storage = models.CharField(choices=ArchiveStorage.choices, help_text='The type of storage used for this archive', max_length=45, blank=True, null=True)
  sealed = models.BooleanField(help_text='Whether or not this archive is sealed (closed to new additions)', default=0)
  owner = CurrentUserField()
  def __str__(self):
    return self.name
  class Meta:
    ordering = ['name']
    verbose_name_plural = "archives"

# Table to catalog of types of battery
class Battery(models.Model):
  name = models.CharField(help_text='Common name of the battery', max_length=45, unique=True)
  voltage = models.DecimalField(help_text='Nominal voltage of the battery', max_digits=5, decimal_places=2, blank=True, null=True)
  chemistry = models.CharField(help_text='Battery chemistry (e.g. Alkaline, Lithium, etc)', max_length=45, blank=True, null=True)
  other_names = models.CharField(help_text='Alternative names for this kind of battery', max_length=45, blank=True, null=True)
  def __str__(self):
    return self.name
  class Meta:
    ordering = ['name']
    verbose_name_plural = "batteries"

# Table to list of physical condition descriptions that can be used to evaluate equipment
class Condition(models.Model):
  code = models.CharField(help_text='Condition shortcode (e.g. EXC)', max_length = 6)
  name = models.CharField(help_text='Full name of condition (e.g. Excellent)', max_length=45)
  min_rating = models.PositiveIntegerField(help_text='The lowest percentage rating that encompasses this condition', validators=[MinValueValidator(0),MaxValueValidator(100)])
  max_rating = models.PositiveIntegerField(help_text='The highest percentage rating that encompasses this condition', validators=[MinValueValidator(0),MaxValueValidator(100)])
  description = models.CharField(help_text='Longer description of condition', max_length=300)
  def __str__(self):
    return "%s - %s" % (self.name, self.description)
  class Meta:
    verbose_name_plural = "conditions"
  
# Exposure programs as defined by EXIF tag ExposureProgram
class ExposureProgram(models.Model):
  name = models.CharField(help_text='Name of exposure program as defined by EXIF tag ExposureProgram', max_length=45) 
  def __str__(self):
    return self.name
  class Meta:
    verbose_name_plural = "exposure programs"

# Table to catalog different protocols used to communicate with flashes
class FlashProtocol(models.Model):
  name = models.CharField(help_text='Name of the flash protocol', max_length=45) 
  manufacturer = models.ForeignKey(Manufacturer, on_delete=models.CASCADE, blank=True, null=True, help_text='Manufacturer who owns this flash protocol')
  def __str__(self):
    if self.manufacturer is not None:
      return "%s %s" % (self.manufacturer.name, self.name)
    else:
      return self.name
  class Meta:
    ordering = ['name']
    verbose_name_plural = "flash protocols"

# Table to catalog filters
class Filter(models.Model):
  type = models.CharField(help_text='Filter type (e.g. Red, CPL, UV)', max_length=45) 
  thread = models.DecimalField(help_text='Diameter of screw thread in mm', max_digits=4, decimal_places=1, blank=True, null=True)
  attenuation = models.DecimalField(help_text='Attenuation of this filter in decimal stops', max_digits=3, decimal_places=1, blank=True, null=True)
  qty = models.PositiveIntegerField(help_text='Quantity of these filters available', default=1, blank=True, null=True)
  manufacturer = models.ForeignKey(Manufacturer, on_delete=models.CASCADE, blank=True, null=True, help_text='Manufacturer of this filter')
  owner = CurrentUserField()
  def __str__(self):
    return "%s %smm" % (self.type, str(self.thread))
  class Meta:
    ordering = ['thread', 'type']
    verbose_name_plural = "filters"

# Table to catalog different negative sizes available. Negtives sizes are distinct from film formats.
class NegativeSize(models.Model):
  name = models.CharField(help_text='Common name of the negative size (e.g. 35mm, 6x7, etc)', max_length=45, unique=True)
  width = models.DecimalField(help_text='Width of the negative size in mm' ,max_digits=4, decimal_places=1, blank=True, null=True)
  height = models.DecimalField(help_text='Height of the negative size in mm', max_digits=4, decimal_places=1, blank=True, null=True)
  crop_factor = models.DecimalField(help_text='Crop factor of this negative size', max_digits=4, decimal_places=2, blank=True, null=True)
  area = models.PositiveIntegerField(help_text='Area of this negative size in sq. mm', blank=True, null=True)
  aspect_ratio = models.DecimalField(help_text='Aspect ratio of this negative size, expressed as a single decimal (e.g. 3:2 is expressed as 1.5)',max_digits=4, decimal_places=2, blank=True, null=True)
  def __str__(self):
    return self.name
  # Override save method to calculate some fields
  def save(self, *args, **kwargs):
    if self.width is not None and self.height is not None:
      self.aspect_ratio = self.width/self.height
      self.area = self.width*self.height
      diag = sqrt(self.width**2 + self.height**2)
      diag35mm = 43.2666
      self.crop_factor = diag35mm/diag
    super().save(*args, **kwargs)
  class Meta:
    ordering = ['area']
    verbose_name_plural = "negative sizes"

# Table to catalogue different film formats. These are distinct from negative sizes.
class Format(models.Model):
  format = models.CharField(help_text='The name of this film/sensor format', max_length=45, unique=True)
  negative_size = models.ManyToManyField(NegativeSize, blank=True)
  def __str__(self):
    return self.format
  class Meta:
    ordering = ['format']
    verbose_name_plural = "formats"

# Table to list all series of cameras and lenses
class Series(models.Model):
  name = models.CharField(help_text='Name of this collection, e.g. Canon FD SLRs', max_length=45, unique=True)
  owner = CurrentUserField()
  def __str__(self):
    return self.name
  class Meta:
    ordering = ['name']
    verbose_name_plural = "series"

# Table to catalog flashes, flashguns and speedlights
class Flash(models.Model):
  model = models.CharField(help_text='Model name/number of the flash', max_length=45)
  manufacturer = models.ForeignKey(Manufacturer, on_delete=models.CASCADE, blank=True, null=True, help_text='Manufacturer of this flash')
  guide_number = models.PositiveIntegerField(help_text='Guide number of the flash', blank=True, null=True)
  gn_info = models.CharField(verbose_name='Guide number info', help_text='Extra freeform info about how the guide number was measured', max_length=45, blank=True, null=True)
  battery_powered = models.BooleanField(help_text='Whether this flash takes batteries', blank=True, null=True)
  pc_sync = models.BooleanField(verbose_name='PC sync', help_text='Whether the flash has a PC sync socket', blank=True, null=True)
  hot_shoe = models.BooleanField(help_text='Whether the flash has a hot shoe connection', blank=True, null=True)
  light_stand = models.BooleanField(help_text='Whether the flash can be used on a light stand', blank=True, null=True)
  battery_type = models.ForeignKey(Battery, on_delete=models.CASCADE, blank=True, null=True, help_text='Type of battery required by this flash')
  battery_qty = models.PositiveIntegerField(help_text='Quantity of batteries needed in this flash', blank=True, null=True)
  manual_control = models.BooleanField(help_text='Whether this flash offers manual power control', blank=True, null=True)
  swivel_head = models.BooleanField(help_text='Whether this flash has a horizontal swivel head', blank=True, null=True)
  tilt_head = models.BooleanField(help_text='Whether this flash has a vertical tilt head', blank=True, null=True)
  zoom = models.BooleanField(help_text='Whether this flash can zoom', blank=True, null=True)
  ttl = models.BooleanField(verbose_name='TTL', help_text='Whether this flash supports TTL metering', blank=True, null=True)
  flash_protocol = models.ForeignKey(FlashProtocol, on_delete=models.CASCADE, blank=True, null=True, help_text='Flash protocol used by this flash')
  trigger_voltage = models.DecimalField(help_text='Trigger voltage of the flash, in Volts', max_digits=5, decimal_places=1, blank=True, null=True)
  own = models.BooleanField(help_text='Whether we currently own this flash', blank=True, null=True)
  acquired = models.DateField(help_text='Date this flash was acquired', blank=True, null=True)
  cost = MoneyField(help_text='Purchase cost of this flash', max_digits=12, decimal_places=2, blank=True, null=True, default_currency='GBP')
  owner = CurrentUserField()
  def __str__(self):
    if self.manufacturer is not None:
      return "%s %s" % (self.manufacturer.name, self.model)
    else:
      return self.model
  class Meta:
    ordering = ['manufacturer', 'model']
    verbose_name_plural = "flashes"
  def clean(self):
    # if battery_type is set, need to supply battery_qty
    if self.battery_type is not None and self.battery_qty is None:
      raise ValidationError({
        'battery_qty': ValidationError(('Must specify number of batteries')),
      })

# Table to list enlargers
class Enlarger(models.Model):
  
  class EnlargerType(DjangoChoices):
    Diffusion = ChoiceItem()
    Condenser = ChoiceItem()

  class LightSource(DjangoChoices):
    Incandescent = ChoiceItem()
    Cold_cathode = ChoiceItem()
    LED = ChoiceItem()

  manufacturer = models.ForeignKey(Manufacturer, on_delete=models.CASCADE, blank=True, null=True, help_text='Manufacturer of this enlarger')
  model = models.CharField(help_text='Name/model of the enlarger', max_length=45)
  negative_size = models.ForeignKey(NegativeSize, on_delete=models.CASCADE, blank=True, null=True, help_text='Largest negative size that this enlarger can accept')
  type = models.CharField(choices=EnlargerType.choices, help_text='The type of optical system in the enlarger', max_length=15, blank=True, null=True)
  light_source = models.CharField(choices=LightSource.choices, help_text='The type of light source used in the enlarger', max_length=15, blank=True, null=True)
  acquired = models.DateField(help_text='Date on which the enlarger was acquired', blank=True, null=True)
  lost = models.DateField(help_text='Date on which the enlarger was lost/sold', blank=True, null=True)
  introduced = models.PositiveIntegerField(help_text='Year in which the enlarger was introduced', blank=True, null=True)
  discontinued = models.PositiveIntegerField(help_text='Year in which the enlarger was discontinued', blank=True, null=True)
  cost = MoneyField(help_text='Purchase cost of this enlarger', max_digits=12, decimal_places=2, blank=True, null=True, default_currency='GBP')
  lost_price = MoneyField(help_text='Sale price of the enlarger', max_digits=12, decimal_places=2, blank=True, null=True, default_currency='GBP')
  owner = CurrentUserField()
  def __str__(self):
    if self.manufacturer is not None:
      return "%s %s" % (self.manufacturer.name, self.model)
    else:
      return self.model
  class Meta:
    ordering = ['manufacturer', 'model']
    verbose_name_plural = "enlargers"
  def clean(self):
    # Acquired/lost
    if self.acquired is not None and self.lost is not None and self.acquired > self.lost:
      raise ValidationError({
        'acquired': ValidationError(('Acquired date must be earlier than lost date')),
        'lost': ValidationError(('Lost date must be later than acquired date')),
      })
    if self.acquired is not None and self.acquired > datetime.date(datetime.now()):
      raise ValidationError({
        'acquired': ValidationError(('Acquired date must be in the past')),
      })
    if self.lost is not None and self.lost > datetime.date(datetime.now()):
      raise ValidationError({
        'lost': ValidationError(('Lost date must be in the past')),
      })
    # Introduced/discontinued
    if self.introduced is not None and self.discontinued is not None and self.introduced > self.discontinued:
      raise ValidationError({
        'introduced': ValidationError(('Introduced date must be earlier than discontinued date')),
        'discontinued': ValidationError(('Discontinued date must be later than introduced date')),
      })
    if self.introduced is not None and self.introduced > datetime.now().year:
      raise ValidationError({
        'introduced': ValidationError(('Introduced date must be in the past')),
      })
    if self.discontinued is not None and self.discontinued > datetime.now().year:
      raise ValidationError({
        'discontinued': ValidationError(('Discontinued date must be in the past')),
      })
  
# Metering modes as defined by EXIF tag MeteringMode
class MeteringMode(models.Model):
  name = models.CharField(help_text='Name of metering mode as defined by EXIF tag MeteringMode', max_length=45)
  def __str__(self):
    return self.name
  class Meta:
    verbose_name_plural = "metering modes"

# Table to catalog different metering technologies and cell types
class MeteringType(models.Model):
  name = models.CharField(help_text='Name of the metering technology (e.g. Selenium)', max_length=45)
  def __str__(self):
    return self.name
  class Meta:
    verbose_name_plural = "metering types"

# Table to catalog different lens mount standards. This is mostly used for camera lens mounts, but can also be used for enlarger and projector lenses.
class Mount(models.Model):

  # Choices for mount types
  class MountType(DjangoChoices):
    Bayonet = ChoiceItem()
    Breech_lock = ChoiceItem()
    Screw = ChoiceItem()
    Friction = ChoiceItem()
    Lens_board = ChoiceItem()

  # Choices for mount purposes
  class Purpose(DjangoChoices):
    Camera = ChoiceItem()
    Enlarger = ChoiceItem()
    Projector = ChoiceItem()
    Telescope = ChoiceItem()
    Microscope = ChoiceItem()

  mount = models.CharField(help_text='Name of this lens mount (e.g. Canon FD)', max_length=45, unique=True)
  shutter_in_lens = models.BooleanField(help_text='Whether this lens mount system incorporates the shutter into the lens', default=0, blank=True, null=True)
  type = models.CharField(help_text='The physical mount type of this lens mount', choices=MountType.choices, max_length=15, blank=True, null=True)
  purpose = models.CharField(help_text='The intended purpose of this lens mount', choices=Purpose.choices, max_length=15, blank=True, null=True)
  notes = models.CharField(help_text='Freeform notes field', max_length=100, blank=True, null=True)
  manufacturer = models.ForeignKey(Manufacturer, on_delete=models.CASCADE, blank=True, null=True, help_text='Manufacturer who owns this lens mount')
  def __str__(self):
    return self.mount
  class Meta:
    ordering = ['mount']
    verbose_name_plural = "mounts"

# Table to catalog different paper stocks available
class PaperStock(models.Model):
  name = models.CharField(help_text='Name of this paper stock', max_length=45)
  manufacturer = models.ForeignKey(Manufacturer, on_delete=models.CASCADE, blank=True, null=True, help_text='Manufacturer of this paper stock')
  resin_coated = models.BooleanField(help_text='Whether the paper is resin-coated', blank=True, null=True)
  tonable = models.BooleanField(help_text='Whether this paper accepts chemical toning', blank=True, null=True)
  colour = models.BooleanField(help_text='Whether this is a colour paper', blank=True, null=True)
  finish = models.CharField(help_text='The finish of the paper surface', max_length=25, blank=True, null=True)
  def __str__(self):
    if self.manufacturer is not None:
      return "%s %s" % (self.manufacturer.name, self.name)
    else:
      return self.name
  class Meta:
    ordering = ['manufacturer', 'name']
    verbose_name_plural = "paper stocks"

# Table to catalog photographers
class Person(models.Model):
  name = models.CharField(help_text='Name of the photographer', max_length=45, unique=True)
  owner = CurrentUserField()
  def __str__(self):
    return self.name
  class Meta:
    ordering = ['name']
    verbose_name_plural = "people"

# Table to catalog chemical processes that can be used to develop film and paper
class Process(models.Model):
  name = models.CharField(help_text='Name of this developmenmt process (e.g. C-41, E-6)', max_length=25, unique=True)
  colour = models.BooleanField(help_text='Whether this is a colour process', blank=True, null=True)
  positive = models.BooleanField(help_text='Whether this is a positive/reversal process', blank=True, null=True)
  def __str__(self):
    return self.name
  class Meta:
    ordering = ['name']
    verbose_name_plural = "processes"

# Table to catalog teleconverters (multipliers)
class Teleconverter(models.Model):
  model = models.CharField(help_text='Model name of this teleconverter', max_length=45)
  manufacturer = models.ForeignKey(Manufacturer, on_delete=models.CASCADE, blank=True, null=True, help_text='Manufacturer of this teleconverter')
  mount = models.ForeignKey(Mount, on_delete=models.CASCADE, blank=True, null=True, help_text='Lens mount used by this teleconverter', limit_choices_to={'purpose': 'Camera'})
  factor = models.DecimalField(help_text='Magnification factor of this teleconverter (numerical part only, e.g. 1.4)', max_digits=4, decimal_places=2, blank=True, null=True)
  elements = models.PositiveIntegerField(help_text='Number of optical elements used in this teleconverter', blank=True, null=True)
  groups = models.PositiveIntegerField(help_text='Number of optical groups used in this teleconverter', blank=True, null=True)
  multicoated = models.BooleanField(help_text='Whether this teleconverter is multi-coated', blank=True, null=True)
  owner = CurrentUserField()
  def __str__(self):
    if self.manufacturer is not None:
      return "%s %s" % (self.manufacturer.name, self.model)
    else:
      return self.model
  class Meta:
    ordering = ['manufacturer', 'model']
    verbose_name_plural = "teleconverters"
  def clean(self):
    # Groups/elements
    if self.groups is not None and self.elements is not None and self.groups > self.elements:
      raise ValidationError({
        'groups': ValidationError(('Cannot have more groups than elements')),
        'elements': ValidationError(('Cannot have fewer elements than groups')),
      })

# Table to catalog paper toners that can be used during the printing process
class Toner(models.Model):
  name = models.CharField(help_text='Name of the toner', max_length=45)
  manufacturer = models.ForeignKey(Manufacturer, on_delete=models.CASCADE, blank=True, null=True, help_text='Manufacturer of this toner')
  formulation = models.CharField(help_text='Chemical formulation of the toner', max_length=45, blank=True, null=True)
  stock_dilution = models.CharField(help_text='Stock dilution of the toner', max_length=10, blank=True, null=True)
  def __str__(self):
    if self.manufacturer is not None:
      return "%s %s" % (self.manufacturer.name, self.name)
    else:
      return self.name
  class Meta:
    ordering = ['manufacturer', 'name']
    verbose_name_plural = "toners"

# Table to list different brands of film stock
class FilmStock(models.Model):
  name = models.CharField(help_text='Name of the filmstock', max_length=45)
  manufacturer = models.ForeignKey(Manufacturer, on_delete=models.CASCADE, blank=True, null=True, help_text='Manufacturer of this film')
  iso = models.PositiveIntegerField(verbose_name='ISO', help_text='Nominal ISO speed of the film', blank=True, null=True)
  colour = models.BooleanField(help_text='Whether the film is colour', blank=True, null=True)
  panchromatic = models.BooleanField(help_text='Whether this film is panchromatic', blank=True, null=True)
  process = models.ForeignKey(Process, on_delete=models.CASCADE, blank=True, null=True, help_text='Development process required by this film')
  def __str__(self):
    if self.manufacturer is not None:
      return "%s %s" % (self.manufacturer.name, self.name)
    else:
      return self.name
  class Meta:
    ordering = ['manufacturer', 'name']
    verbose_name_plural = "film stocks"

# Table to record bulk film stock, from which individual films can be cut
class BulkFilm(models.Model):
  format = models.ForeignKey(Format, on_delete=models.CASCADE, help_text='Film format of this bulk film')
  filmstock = models.ForeignKey(FilmStock, on_delete=models.CASCADE, help_text='Filmstock of this bulk film')
  purchase_date = models.DateField(help_text='Purchase date of this bulk roll', blank=True, null=True)
  cost = MoneyField(help_text='Purchase cost of this bulk roll', max_digits=12, decimal_places=2, blank=True, null=True, default_currency='GBP')
  source = models.CharField(help_text='Place where this bulk roll was bought from', max_length=45, blank=True, null=True)
  batch = models.CharField(help_text='Batch code of this bulk roll', max_length=45, blank=True, null=True)
  expiry = models.DateField(help_text='Expiry date of this bulk roll', blank=True, null=True)
  owner = CurrentUserField()
  def __str__(self):
    return "#%s %s %s" % (self.pk, self.filmstock.manufacturer.name, self.filmstock.name)
  class Meta:
    verbose_name_plural = "bulk films"

# Table to catalog adapters to mount lenses on other cameras
class MountAdapter(models.Model):
  camera_mount = models.ForeignKey(Mount, on_delete=models.CASCADE, help_text='Mount used to attach a camera', related_name="camera_mount")
  lens_mount = models.ForeignKey(Mount, on_delete=models.CASCADE, help_text='Mount used to attach a lens', related_name="lens_mount")
  has_optics = models.BooleanField(help_text='Whether this adapter includes optical elements', blank=True, null=True)
  infinity_focus = models.BooleanField(help_text='Whether this adapter allows infinity focus', blank=True, null=True)
  notes = models.CharField(help_text='Freeform notes', max_length=100, blank=True, null=True)
  owner = CurrentUserField()
  def __str__(self):
    return "%s - %s" % (self.camera_mount, self.lens_mount)
  class Meta:
    ordering = ['camera_mount', 'lens_mount']
    verbose_name_plural = "mount adapters"

# Table to list all possible shutter speeds
class ShutterSpeed(models.Model):
  shutter_speed = models.CharField(help_text='Shutter speed in fractional notation, e.g. 1/250', max_length=10, primary_key=True)
  # field validation: like 1/500 or 2
  duration = models.DecimalField(help_text='Shutter speed in decimal notation, e.g. 0.04', max_digits=9, decimal_places=5)
  def __str__(self):
    return self.shutter_speed
  def save(self, *args, **kwargs):
    # Test if format is 1/125
    m0 = re.match(r'^(\d{1})/(\d+)$', self.shutter_speed)
    # Test if format is 1 or 1"
    m1 = re.match(r'^(\d+)"?$', self.shutter_speed)
    if m0:
      self.duration = int(m0.group(1)) / int(m0.group(2))
    elif m1:
      self.duration = m1.group(1)
    super().save(*args, **kwargs)
  class Meta:
    ordering = ['duration']
    verbose_name_plural = "shutter speeds"

# Table to list film and paper developers
class Developer(models.Model):
  manufacturer = models.ForeignKey(Manufacturer, on_delete=models.CASCADE, blank=True, null=True, help_text='Manufacturer of this developer')
  name = models.CharField(help_text='Name of the developer', max_length=45)
  for_paper = models.BooleanField(help_text='Whether this developer can be used with paper', blank=True, null=True)
  for_film = models.BooleanField(help_text='Whether this developer can be used with film', blank=True, null=True)
  chemistry = models.CharField(help_text='The key chemistry on which this developer is based (e.g. phenidone)', max_length=45, blank=True, null=True)
  def __str__(self):
    if self.manufacturer is not None:
      return "%s %s" % (self.manufacturer.name, self.name)
    else:
      return self.name
  class Meta:
    ordering = ['manufacturer', 'name']
    verbose_name_plural = "developers"

# Table to catalog lens models
class LensModel(models.Model):
  manufacturer = models.ForeignKey(Manufacturer, on_delete=models.CASCADE, blank=True, null=True, help_text='Manufacturer of this lens model')
  model = models.CharField(help_text='Model name of this lens', max_length=45)
  mount = models.ForeignKey(Mount, on_delete=models.CASCADE, blank=True, null=True, help_text='Mount used by this lens model')
  zoom = models.BooleanField(help_text='Whether this is a zoom lens', blank=True, null=True)
  min_focal_length = models.PositiveIntegerField(help_text='Shortest focal length of this lens, in mm', blank=True, null=True)
  max_focal_length = models.PositiveIntegerField(help_text='Longest focal length of this lens, in mm', blank=True, null=True)
  closest_focus = models.PositiveIntegerField(help_text='The closest focus possible with this lens, in cm', blank=True, null=True)
  max_aperture = models.DecimalField(help_text='Maximum (widest) aperture available on this lens (numerical part only, e.g. 2.8)', max_digits=4, decimal_places=1, blank=True, null=True)
  min_aperture = models.DecimalField(help_text='Minimum (narrowest) aperture available on this lens (numerical part only, e.g. 22)', max_digits=4, decimal_places=1, blank=True, null=True)
  elements = models.PositiveIntegerField(help_text='Number of optical lens elements', blank=True, null=True)
  groups = models.PositiveIntegerField(help_text='Number of optical groups', blank=True, null=True)
  weight = models.PositiveIntegerField(help_text='Weight of this lens, in grammes (g)', blank=True, null=True)
  nominal_min_angle_diag = models.PositiveIntegerField(verbose_name='Min angle of view', help_text='Nominal minimum diagonal field of view from manufacturer specs', blank=True, null=True, validators=[MinValueValidator(0),MaxValueValidator(360)])
  nominal_max_angle_diag = models.PositiveIntegerField(verbose_name='Max angle of view', help_text='Nominal maximum diagonal field of view from manufacturer specs', blank=True, null=True, validators=[MinValueValidator(0),MaxValueValidator(360)])
  aperture_blades = models.PositiveIntegerField(help_text='Number of aperture blades', blank=True, null=True)
  autofocus = models.BooleanField(help_text='Whether this lens has autofocus capability', blank=True, null=True)
  filter_thread = models.DecimalField(help_text='Diameter of lens filter thread, in mm', max_digits=4, decimal_places=1, blank=True, null=True)
  magnification = models.DecimalField(help_text='Maximum magnification ratio of the lens, expressed like 0.765', max_digits=5, decimal_places=3, blank=True, null=True)
  url = models.URLField(help_text='URL to more information about this lens', blank=True, null=True)
  introduced = models.PositiveIntegerField(help_text='Year in which this lens model was introduced', blank=True, null=True)
  discontinued = models.PositiveIntegerField(help_text='Year in which this lens model was discontinued', blank=True, null=True)
  negative_size = models.ForeignKey(NegativeSize, on_delete=models.CASCADE, blank=True, null=True, help_text='Largest negative size that this lens is designed for')
  fixed_mount = models.BooleanField(help_text='Whether this is a fixed lens (i.e. on a compact camera)', blank=True, null=True)
  notes = models.CharField(help_text='Freeform notes field', max_length=100, blank=True, null=True)
  coating = models.CharField(help_text='Notes about the lens coating type', max_length=45, blank=True, null=True)
  hood = models.CharField(help_text='Model number of the compatible lens hood', max_length=45, blank=True, null=True)
  exif_lenstype = models.CharField(help_text='EXIF LensID number, if this lens has one officially registered. See documentation at http://www.sno.phy.queensu.ca/~phil/exiftool/TagNames/', max_length=45, blank=True, null=True)
  rectilinear = models.BooleanField(help_text='Whether this is a rectilinear lens', default=1, blank=True, null=True)
  length = models.PositiveIntegerField(help_text='Length of lens in mm', blank=True, null=True)
  diameter = models.PositiveIntegerField(help_text='Width of lens in mm', blank=True, null=True)
  image_circle = models.PositiveIntegerField(help_text='Diameter of image circle projected by lens, in mm', blank=True, null=True)
  formula = models.CharField(help_text='Name of the type of lens formula (e.g. Tessar)', max_length=45, blank=True, null=True)
  shutter_model = models.CharField(help_text='Name of the integrated shutter, if any', max_length=45, blank=True, null=True)
  series = models.ManyToManyField(Series, blank=True)
  def __str__(self):
    if self.manufacturer is not None:
      return "%s %s" % (self.manufacturer.name, self.model)
    else:
      return self.model
  class Meta:
    ordering = ['manufacturer', 'model']
    verbose_name_plural = "lens models"

  def clean(self):
    # Check focal length
    if self.min_focal_length is not None and self.max_focal_length is not None and self.min_focal_length > self.max_focal_length:
      raise ValidationError({
        'min_focal_length': ValidationError(('Min focal length must be smaller than max focal length')),
        'max_focal_length': ValidationError(('Max focal length must be larger than min focal length')),
      })

    # Angle of view
    if self.nominal_min_angle_diag is not None and self.nominal_max_angle_diag is not None and self.nominal_min_angle_diag > self.nominal_max_angle_diag:
      raise ValidationError({
        'nominal_min_angle_diag': ValidationError(('Min angle of view must be smaller than max angle of view')),
        'nominal_max_angle_diag': ValidationError(('Max angle of view must be larger than min angle of view')),
      })

    # Introduced/discontinued
    if self.introduced is not None and self.discontinued is not None and self.introduced > self.discontinued:
      raise ValidationError({
        'introduced': ValidationError(('Introduced date must be earlier than discontinued date')),
        'discontinued': ValidationError(('Discontinued date must be later than introduced date')),
      })
    if self.introduced is not None and self.introduced > datetime.now().year:
      raise ValidationError({
        'introduced': ValidationError(('Introduced date must be in the past')),
      })
    if self.discontinued is not None and self.discontinued > datetime.now().year:
      raise ValidationError({
        'discontinued': ValidationError(('Discontinued date must be in the past')),
      })

    # Groups and elements
    if self.groups is not None and self.elements is not None and self.elements < self.groups:
      raise ValidationError({
        'elements': ValidationError(("Can't have more groups than elements")),
        'groups': ValidationError(("Can't have more groups than elements")),
      })

    # Fixed mount vs mount ID
    if self.fixed_mount == True and self.mount is not None:
      raise ValidationError({'mount': 'Do not choose a mount when fixed mount is true'})

    # Zoom lenses
    if self.zoom == False and self.min_focal_length and self.max_focal_length and self.min_focal_length != self.max_focal_length:
      raise ValidationError({
        'min_focal_length': ValidationError(('Min and max focal lengths must be equal for non-zoom lenses')),
        'max_focal_length': ValidationError(('Min and max focal lengths must be equal for non-zoom lenses')),
      })

    # Aperture range
    if self.max_aperture is not None and self.min_aperture is not None and self.max_aperture > self.min_aperture:
      raise ValidationError({
        'max_aperture': ValidationError(('Max aperture must be smaller than min aperture')),
        'min_aperture': ValidationError(('Max aperture must be smaller than min aperture')),
      })

  def save(self, *args, **kwargs):
    # Auto-populate focal length
    if self.zoom is False and self.min_focal_length is not None:
      self.max_focal_length = self.min_focal_length
    super().save(*args, **kwargs)

# Table to catalog camera models - both cameras with fixed and interchangeable lenses
class CameraModel(models.Model):
  # Choices for body types
  class BodyType(DjangoChoices):
    Box_camera = ChoiceItem()
    Folding_camera = ChoiceItem()
    Compact_camera = ChoiceItem()
    SLR = ChoiceItem()
    TLR = ChoiceItem()
    Bridge_camera = ChoiceItem()
    View_camera = ChoiceItem()
    Pistol_grip_camera = ChoiceItem()
    Miniature_camera = ChoiceItem()

  # Choices for focus type
  class FocusType(DjangoChoices):
    Autofocus = ChoiceItem()
    Fixed_focus = ChoiceItem()
    Zone_focus = ChoiceItem()
    Rangefinder = ChoiceItem()
    SLR = ChoiceItem()
    TLR = ChoiceItem()
    View_camera = ChoiceItem()

  # Choices for shutter type
  class ShutterType(DjangoChoices):
    Focal_plane_cloth = ChoiceItem()
    Focal_plane_metal = ChoiceItem()
    Leaf = ChoiceItem()
    Rotary = ChoiceItem()
    Sliding = ChoiceItem()
    Electronic = ChoiceItem()

  manufacturer = models.ForeignKey(Manufacturer, on_delete=models.CASCADE, blank=True, null=True, help_text='Manufacturer of this camera model')
  model = models.CharField(help_text='The model name of the camera', max_length=45)
  mount = models.ForeignKey(Mount, on_delete=models.CASCADE, blank=True, null=True, help_text='Lens mount used by this camera model', limit_choices_to={'purpose': 'Camera'})
  format = models.ForeignKey(Format, on_delete=models.CASCADE, blank=True, null=True, help_text='Film format used by this camera model')
  focus_type = models.CharField(choices=FocusType.choices, max_length=25, blank=True, null=True, help_text='Focus type used on this camera model')
  metering = models.BooleanField(help_text='Whether the camera has built-in metering', blank=True, null=True)
  coupled_metering = models.BooleanField(help_text='Whether the camera''s meter is coupled automatically', blank=True, null=True)
  metering_type = models.ForeignKey(MeteringType, on_delete=models.CASCADE, blank=True, null=True, help_text='Metering type used on this camera model')
  introduced = models.PositiveIntegerField(help_text='Year in which the camera model was introduced', blank=True, null=True)
  discontinued = models.PositiveIntegerField(help_text='Year in which the camera model was discontinued', blank=True, null=True)
  body_type = models.CharField(choices=BodyType.choices, max_length=25, blank=True, null=True, help_text='Body type of this camera model')
  weight = models.PositiveIntegerField(help_text='Weight of the camera body (without lens or batteries) in grammes (g)', blank=True, null=True)
  introduced = models.PositiveIntegerField(help_text='Year in which the camera model was introduced', blank=True, null=True)
  discontinued = models.PositiveIntegerField(help_text='Year in which the camera model was discontinued', blank=True, null=True)
  negative_size = models.ForeignKey(NegativeSize, on_delete=models.CASCADE, blank=True, null=True, help_text='Size of negative created by this camera')
  shutter_type = models.CharField(choices=ShutterType.choices, max_length=25, blank=True, null=True, help_text='Type of shutter used on this camera model')
  shutter_model = models.CharField(help_text='Model of shutter', max_length=45, blank=True, null=True)
  cable_release = models.BooleanField(help_text='Whether the camera has the facility for a remote cable release', blank=True, null=True)
  viewfinder_coverage = models.PositiveIntegerField(help_text='Percentage coverage of the viewfinder. Mostly applicable to SLRs.', blank=True, null=True, validators=[MinValueValidator(0),MaxValueValidator(100)])
  power_drive = models.BooleanField(help_text='Whether the camera has integrated motor drive', blank=True, null=True)
  continuous_fps = models.DecimalField(help_text='The maximum rate at which the camera can shoot, in frames per second', max_digits=4, decimal_places=1, blank=True, null=True)
  fixed_mount = models.BooleanField(help_text='Whether the camera has a fixed lens', blank=True, null=True)
  lensmodel = models.ForeignKey(LensModel, on_delete=models.CASCADE, blank=True, null=True, help_text='Lens model attached to this camera model, if it is a fixed-lens camera', limit_choices_to={'fixed_mount': True})
  battery_qty = models.PositiveIntegerField(help_text='Quantity of batteries needed', blank=True, null=True)
  battery_type = models.ForeignKey(Battery, on_delete=models.CASCADE, blank=True, null=True, help_text='Battery type that this camera model needs')
  notes = models.CharField(help_text='Freeform text field for extra notes', max_length=100, blank=True, null=True)
  bulb = models.BooleanField(help_text='Whether the camera supports bulb (B) exposure', blank=True, null=True)
  time = models.BooleanField(help_text='Whether the camera supports time (T) exposure', blank=True, null=True)
  min_iso = models.PositiveIntegerField(verbose_name='Min ISO', help_text='Minimum ISO the camera will accept for metering', blank=True, null=True)
  max_iso = models.PositiveIntegerField(verbose_name='Max ISO', help_text='Maximum ISO the camera will accept for metering', blank=True, null=True)
  af_points = models.PositiveIntegerField(verbose_name='Autofocus points', help_text='Number of autofocus points', blank=True, null=True)
  int_flash = models.BooleanField(verbose_name='Internal flash', help_text='Whether the camera has an integrated flash', blank=True, null=True)
  int_flash_gn = models.PositiveIntegerField(verbose_name='Internal flash guide number', help_text='Guide number of internal flash', blank=True, null=True)
  ext_flash = models.BooleanField(verbose_name='External flash', help_text='Whether the camera supports an external flash', blank=True, null=True)
  flash_metering = models.ForeignKey(FlashProtocol, on_delete=models.CASCADE, blank=True, null=True, help_text='Whether this camera model supports flash metering')
  pc_sync = models.BooleanField(verbose_name='PC sync', help_text='Whether the camera has a PC sync socket for flash', blank=True, null=True)
  hotshoe = models.BooleanField(help_text='Whether the camera has a hotshoe', blank=True, null=True)
  coldshoe = models.BooleanField(help_text='Whether the camera has a coldshoe or accessory shoe', blank=True, null=True)
  #x_sync = models.ForeignKey(ShutterSpeed, on_delete=models.CASCADE, blank=True, null=True)
  meter_min_ev = models.PositiveIntegerField(verbose_name='Min EV', help_text='Lowest EV/LV the built-in meter supports', blank=True, null=True)
  meter_max_ev = models.PositiveIntegerField(verbose_name='Max EV', help_text='Highest EV/LV the built-in meter supports', blank=True, null=True)
  dof_preview = models.BooleanField(verbose_name='DoF preview', help_text='Whether the camera has depth of field preview', blank=True, null=True)
  tripod = models.BooleanField(help_text='Whether the camera has a tripod bush', blank=True, null=True)
  shutter_speeds = models.ManyToManyField(ShutterSpeed, blank=True)
  metering_modes = models.ManyToManyField(MeteringMode, blank=True)
  exposure_programs = models.ManyToManyField(ExposureProgram, blank=True)
  series = models.ManyToManyField(Series, blank=True)
  def __str__(self):
    if self.manufacturer is not None:
      return "%s %s" % (self.manufacturer.name, self.model)
    else:
      return self.model
  class Meta:
    ordering = ['manufacturer', 'model']
    verbose_name_plural = "camera models"

  def clean(self):
    # ISO
    if self.min_iso is not None and self.max_iso is not None and self.min_iso > self.max_iso:
      raise ValidationError({
        'min_iso': ValidationError(('Min ISO must be smaller than max ISO')),
        'max_iso': ValidationError(('Max ISO must be larger than min ISO')),
      })
    # EV
    if self.meter_min_ev is not None and self.meter_max_ev is not None and self.meter_min_ev > self.meter_max_ev:
      raise ValidationError({
        'meter_min_ev': ValidationError(('Min EV must be smaller than max EV')),
        'meter_max_ev': ValidationError(('Max EV must be larger than min EV')),
      })
    # Introduced/discontinued
    if self.introduced is not None and self.discontinued is not None and self.introduced > self.discontinued:
      raise ValidationError({
        'introduced': ValidationError(('Introduced date must be earlier than discontinued date')),
        'discontinued': ValidationError(('Discontinued date must be later than introduced date')),
      })
    if self.introduced is not None and self.introduced > datetime.now().year:
      raise ValidationError({
        'introduced': ValidationError(('Introduced date must be in the past')),
      })
    if self.discontinued is not None and self.discontinued > datetime.now().year:
      raise ValidationError({
        'discontinued': ValidationError(('Discontinued date must be in the past')),
      })
    # Metering bools
    if self.metering is not None and self.metering is False:
      if self.coupled_metering is True:
        raise ValidationError({
          'couples_metering': ValidationError(('Cannot set coupled metering if camera model has no metering')),
        })
      if self.metering_type is True:
        raise ValidationError({
          'discontinued': ValidationError(('Cannot set metering type if camera model has no metering')),
        })
      if self.metering_modes is True:
        raise ValidationError({
          'discontinued': ValidationError(('Cannot set metering modes if camera model has no metering')),
        })
    # int_flash_gn
    if self.int_flash is False and self.int_flash_gn is not None:
      raise ValidationError({
        'int_flash_gn': ValidationError(('Cannot set internal flash guide number if camera model has no internal flash')),
      })

# Table to catalog accessories that are not tracked in more specific tables
class Accessory(models.Model):
  # Choices for accessory types
  class AccessoryType(DjangoChoices):
    Battery_grip = ChoiceItem()
    Case = ChoiceItem()
    Film_back = ChoiceItem()
    Focusing_screen = ChoiceItem()
    Lens_hood = ChoiceItem()
    Lens_cap = ChoiceItem()
    Power_winder = ChoiceItem()
    Viewfinder = ChoiceItem()
    Rangefinder = ChoiceItem()
    Projector = ChoiceItem()
    Light_meter = ChoiceItem()

  type = models.CharField(choices=AccessoryType.choices, help_text='Type of accessory', max_length=15)
  manufacturer = models.ForeignKey(Manufacturer, on_delete=models.CASCADE, blank=True, null=True, help_text='Manufacturer of this accessory')
  model = models.CharField(help_text='Model of the accessory', max_length=45)
  acquired = models.DateField(help_text='Date that this accessory was acquired', blank=True, null=True)
  cost = MoneyField(help_text='Purchase cost of the accessory', max_digits=12, decimal_places=2, blank=True, null=True, default_currency='GBP')
  lost = models.DateField(help_text='Date that this accessory was lost', blank=True, null=True)
  lost_price = MoneyField(help_text='Sale price of the accessory', max_digits=12, decimal_places=2, blank=True, null=True, default_currency='GBP')
  camera_model_compatibility = models.ManyToManyField(CameraModel, blank=True)
  lens_model_compatibility = models.ManyToManyField(LensModel, blank=True)
  owner = CurrentUserField()
  def __str__(self):
    if self.manufacturer is not None:
      return "%s %s" % (self.manufacturer.name, self.model)
    else:
      return self.model
  class Meta:
    ordering = ['manufacturer', 'model']
    verbose_name_plural = "accessories"
  def clean(self):
    # Acquired/lost
    if self.acquired is not None and self.lost is not None and self.acquired > self.lost:
      raise ValidationError({
        'acquired': ValidationError(('Acquired date must be earlier than lost date')),
        'lost': ValidationError(('Lost date must be later than acquired date')),
      })
    if self.acquired is not None and self.acquired > datetime.date(datetime.now()):
      raise ValidationError({
        'acquired': ValidationError(('Acquired date must be in the past')),
      })
    if self.lost is not None and self.lost > datetime.date(datetime.now()):
      raise ValidationError({
        'lost': ValidationError(('Lost date must be in the past')),
      })

# Table to catalog lenses
class Lens(models.Model):
  lensmodel = models.ForeignKey(LensModel, on_delete=models.CASCADE, help_text='Lens model of this lens')
  serial = models.CharField(help_text='Serial number of this lens', max_length=45, blank=True, null=True)
  date_code = models.CharField(help_text='Date code of this lens, if different from the serial number', max_length=45, blank=True, null=True)
  manufactured = models.PositiveIntegerField(help_text='Year in which this specific lens was manufactured', blank=True, null=True)
  acquired = models.DateField(help_text='Date on which this lens was acquired', blank=True, null=True)
  cost = MoneyField(help_text='Price paid for this lens', max_digits=12, decimal_places=2, blank=True, null=True, default_currency='GBP')
  notes = models.CharField(help_text='Freeform notes field', max_length=45, blank=True, null=True)
  own = models.BooleanField(help_text='Whether we currently own this lens', blank=True, null=True)
  lost = models.DateField(help_text='Date on which lens was lost/sold/disposed', blank=True, null=True)
  lost_price = MoneyField(help_text='Sale price of the lens', max_digits=12, decimal_places=2, blank=True, null=True, default_currency='GBP')
  source = models.CharField(help_text='Place where the lens was acquired from', max_length=150, blank=True, null=True)
  condition = models.ForeignKey(Condition, on_delete=models.CASCADE, blank=True, null=True, help_text='Condition of this lens')
  condition_notes = models.CharField(help_text='Description of condition', max_length=150, blank=True, null=True)
  owner = CurrentUserField()
  def __str__(self):
    return "%s %s (#%s)" % (self.lensmodel.manufacturer.name, self.lensmodel.model, self.serial)
  class Meta:
    ordering = ['lensmodel__manufacturer', 'lensmodel__model', 'serial']
    verbose_name_plural = "lenses"
    unique_together = ['lensmodel', 'serial']
  def clean(self):
    if self.acquired is not None and self.lost is not None and self.acquired > self.lost:
      raise ValidationError({
        'acquired': ValidationError(('Acquired date must be earlier than lost date')),
        'lost': ValidationError(('Lost date must be later than acquired date')),
      })
    if self.acquired is not None and self.acquired > datetime.date(datetime.now()):
      raise ValidationError({
        'acquired': ValidationError(('Acquired date must be in the past')),
      })
    if self.lost is not None and self.lost > datetime.date(datetime.now()):
      raise ValidationError({
        'lost': ValidationError(('Lost date must be in the past')),
      })
    # Manufactured date must be in range of introduced-discontinued of the model
    if self.manufactured is not None:
      if self.lensmodel.introduced is not None and self.manufactured < self.lensmodel.introduced:
        raise ValidationError({
          'manufactured': ValidationError(('Manufactured date cannot be earlier than the date the lens model was introduced')),
        })
      if self.lensmodel.discontinued is not None and self.manufactured < self.lensmodel.discontinued:
        raise ValidationError({
          'manufactured': ValidationError(('Manufactured date cannot be later than the date the lens model was discontinued')),
        })

# Table to catalog cameras - both cameras with fixed lenses and cameras with interchangeable lenses
class Camera(models.Model):
  cameramodel = models.ForeignKey(CameraModel, on_delete=models.CASCADE, help_text='Camera model of this camera')
  acquired = models.DateField(help_text='Date on which the camera was acquired', blank=True, null=True)
  cost = MoneyField(help_text='Price paid for this camera', max_digits=12, decimal_places=2, blank=True, null=True, default_currency='GBP')
  serial = models.CharField(help_text='Serial number of the camera', max_length=45, blank=True, null=True)
  datecode = models.CharField(help_text='Date code of the camera, if different from the serial number', max_length=45, blank=True, null=True)
  manufactured = models.PositiveIntegerField(help_text='Year of manufacture of the camera', blank=True, null=True)
  own = models.BooleanField(help_text='Whether the camera is currently owned', blank=True, null=True)
  lens = models.ForeignKey(Lens, on_delete=models.CASCADE, blank=True, null=True, help_text='Lens attached to this camera, if it is a fixed-lens camera', limit_choices_to={'lensmodel__fixed_mount': True})
  notes = models.CharField(help_text='Freeform text field for extra notes', max_length=100, blank=True, null=True)
  lost = models.DateField(help_text='Date on which the camera was lost/sold/etc', blank=True, null=True)
  lost_price = MoneyField(help_text='Sale price of the camera', max_digits=12, decimal_places=2, blank=True, null=True, default_currency='GBP')
  source = models.CharField(help_text='Where the camera was acquired from', max_length=150, blank=True, null=True)
  condition = models.ForeignKey(Condition, on_delete=models.CASCADE, blank=True, null=True, help_text='Condition of this camera')
  condition_notes = models.CharField(help_text='Description of condition', max_length=150, blank=True, null=True)
  display_lens = models.OneToOneField(Lens, on_delete=models.CASCADE, blank=True, null=True, help_text='Lens that this camera should be displayed with', related_name='display_camera')
  owner = CurrentUserField()
  def __str__(self):
    return "%s %s (#%s)" % (self.cameramodel.manufacturer.name, self.cameramodel.model, self.serial)
  class Meta:
    ordering = ['cameramodel__manufacturer', 'cameramodel__model', 'serial']
    verbose_name_plural = "cameras"
    unique_together = ['cameramodel', 'serial']
  def clean(self):
    if self.acquired is not None and self.lost is not None and self.acquired > self.lost:
      raise ValidationError({
        'acquired': ValidationError(('Acquired date must be earlier than lost date')),
        'lost': ValidationError(('Lost date must be later than acquired date')),
      })
    if self.acquired is not None and self.acquired > datetime.date(datetime.now()):
      raise ValidationError({
        'acquired': ValidationError(('Acquired date must be in the past')),
      })
    if self.lost is not None and self.lost > datetime.date(datetime.now()):
      raise ValidationError({
        'lost': ValidationError(('Lost date must be in the past')),
      })
    # Manufactured date must be in range of introduced-discontinued of the model
    if self.manufactured is not None:
      if self.cameramodel.introduced is not None and self.manufactured < self.cameramodel.introduced:
        raise ValidationError({
          'manufactured': ValidationError(('Manufactured date cannot be earlier than the date the camera model was introduced')),
        })
      if self.cameramodel.discontinued is not None and self.manufactured < self.cameramodel.discontinued:
        raise ValidationError({
          'manufactured': ValidationError(('Manufactured date cannot be later than the date the camera model was discontinued')),
        })

# Table to list films which consist of one or more negatives. A film can be a roll film, one or more sheets of sheet film, one or more photographic plates, etc.
class Film(models.Model):
  filmstock = models.ForeignKey(FilmStock, on_delete=models.CASCADE, help_text='Filmstock that this film is')
  exposed_at = models.PositiveIntegerField(help_text='ISO at which the film was exposed', blank=True, null=True)
  format = models.ForeignKey(Format, on_delete=models.CASCADE, help_text='Film format of this film')
  date_loaded = models.DateField(help_text='Date when the film was loaded into a camera', blank=True, null=True)
  date_processed = models.DateField(help_text='Date when the film was processed', blank=True, null=True)
  camera = models.ForeignKey(Camera, on_delete=models.CASCADE, blank=True, null=True, help_text='Camera that this film was loaded into')
  title = models.CharField(help_text='Title of the film', max_length=150, blank=True, null=True)
  frames = models.PositiveIntegerField(help_text='Expected (not actual) number of frames from the film', blank=True, null=True)
  developer = models.ForeignKey(Developer, on_delete=models.CASCADE, blank=True, null=True, help_text='Developer used to develop this film', limit_choices_to={'for_film': True})
  directory = models.CharField(help_text='Name of the directory that contains the scanned images from this film', max_length=100, blank=True, null=True)
  dev_uses = models.PositiveIntegerField(help_text='Number of previous uses of the developer', blank=True, null=True)
  dev_time = models.DurationField(help_text='Duration of development', blank=True, null=True)
  dev_temp = models.DecimalField(help_text='Temperature of development', max_digits=3, decimal_places=1, blank=True, null=True)
  dev_n = models.IntegerField(help_text='Number of the Push/Pull rating of the film, e.g. N+1, N-2', blank=True, null=True)
  development_notes = models.CharField(help_text='Extra freeform notes about the development process', max_length=200, blank=True, null=True)
  bulk_film = models.ForeignKey(BulkFilm, on_delete=models.CASCADE, blank=True, null=True, help_text='Bulk film this film was cut from')
  bulk_film_loaded = models.DateField(help_text='Date that this film was cut from a bulk roll', blank=True, null=True)
  film_batch = models.CharField(help_text='Batch number of the film', max_length=45, blank=True, null=True)
  expiry_date = models.DateField(help_text='Expiry date of the film', blank=True, null=True)
  purchase_date = models.DateField(help_text='Date this film was purchased', blank=True, null=True)
  price = MoneyField(help_text='Price paid for this film', max_digits=12, decimal_places=2, blank=True, null=True, default_currency='GBP')
  processed_by = models.ForeignKey(Person, on_delete=models.CASCADE, blank=True, null=True, help_text='Person or place that processed this film')
  archive = models.ForeignKey(Archive, on_delete=models.CASCADE, blank=True, null=True, help_text='Archive that this film is stored in')
  owner = CurrentUserField()
  def __str__(self):
    return "#%i %s" % (self.pk, self.title)
  class Meta:
    verbose_name_plural = "films"
  def clean(self):
    # Date constraints
    if self.date_loaded is not None and self.date_processed is not None and self.date_loaded > self.date_processed:
      raise ValidationError({
        'date_loaded': ValidationError(('Date loaded cannot be later than the date the film was processed')),
        'date_processed': ValidationError(('Date processed cannot be earlier than the date the film was loaded')),
      })
  def save(self, *args, **kwargs):
    # Auto-populate values from bulk films
    if self.bulk_film:
      if self.bulk_film.expiry:
        self.expiry_date = self.bulk_film.expiry
      if self.bulk_film.batch:
        self.film_batch = self.bulk_film.batch
      if self.bulk_film.purchase_date:
        self.purchase_date = self.bulk_film.purchase_date
    super().save(*args, **kwargs)

# Table to catalog negatives (including positives/slides). Negatives are created by cameras, belong to films and can be used to create scans or prints.
class Negative(models.Model):
  film = models.ForeignKey(Film, on_delete=models.CASCADE, help_text='Film that this negative belongs to')
  frame = models.CharField(help_text='Frame number or code of this negative', max_length=8)
  caption = models.CharField(help_text='Caption of this picture', max_length=150, blank=True, null=True)
  date = models.DateTimeField(help_text='Date & time on which this picture was taken', blank=True, null=True)
  lens = models.ForeignKey(Lens, on_delete=models.CASCADE, blank=True, null=True, help_text='Lens used to take this negative')
  shutter_speed = models.ForeignKey(ShutterSpeed, on_delete=models.CASCADE, blank=True, null=True, help_text='Shutter speed used to take this negative')
  aperture = models.DecimalField(help_text='Aperture used to take this picture (numerical part only)', max_digits=4, decimal_places=1, blank=True, null=True)
  filter = models.ForeignKey(Filter, on_delete=models.CASCADE, blank=True, null=True, help_text='Filter used when taking this negative')
  teleconverter = models.ForeignKey(Teleconverter, on_delete=models.CASCADE, blank=True, null=True, help_text='Teleconverter used when taking this negative')
  notes = models.CharField(help_text='Extra freeform notes about this exposure', max_length=200, blank=True, null=True)
  mount_adapter = models.ForeignKey(MountAdapter, on_delete=models.CASCADE, blank=True, null=True, help_text='Mount adapter used to mount lens')
  focal_length = models.PositiveIntegerField(help_text='If a zoom lens was used, specify the focal length of the lens', blank=True, null=True)
  latitude = models.DecimalField(help_text='Latitude of the location where the picture was taken', max_digits=9, decimal_places=6, blank=True, null=True, validators=[MinValueValidator(-90),MaxValueValidator(90)])
  longitude = models.DecimalField(help_text='Longitude of the location where the picture was taken', max_digits=9, decimal_places=6, blank=True, null=True, validators=[MinValueValidator(-180),MaxValueValidator(180)])
  flash = models.BooleanField(help_text='Whether flash was used', blank=True, null=True)
  metering_mode = models.ForeignKey(MeteringMode, on_delete=models.CASCADE, blank=True, null=True, help_text='Metering mode used when taking the image')
  exposure_program = models.ForeignKey(ExposureProgram, on_delete=models.CASCADE, blank=True, null=True, help_text='Exposure program used when taking the image')
  photographer = models.ForeignKey(Person, on_delete=models.CASCADE, blank=True, null=True, help_text='Photographer who took the negative')
  copy_of = models.ForeignKey('self', on_delete=models.CASCADE, blank=True, null=True, related_name='copy', help_text='Negative that this was duplicated from')
  owner = CurrentUserField()
  def __str__(self):
    return "%s/%s %s" % (self.film.pk, self.frame, self.caption)
  class Meta:
    ordering = ['film', 'frame']
    verbose_name_plural = "negatives"
    unique_together = ['film', 'frame']
  def clean(self):
    # Aperture must be in range of lens model aperture
    if self.aperture is not None and self.lens is not None:
      if self.lens.lensmodel.max_aperture is not None and self.aperture < self.lens.lensmodel.max_aperture:
        raise ValidationError({
          'aperture': ValidationError(('Aperture cannot be greater than the maximum aperture of the lens')),
        })
      if self.lens.lensmodel.min_aperture is not None and self.aperture > self.lens.lensmodel.min_aperture:
        raise ValidationError({
          'aperture': ValidationError(('Aperture cannot be smaller than the minimum aperture of the lens')),
        })
    # Focal length must be in range of lens model fl
    if self.focal_length is not None and self.lens is not None:
      if self.lens.lensmodel.min_focal_length is not None and self.focal_length < self.lens.lensmodel.min_focal_length:
        raise ValidationError({
          'focal_length': ValidationError(('Focal length cannot be shorter than the minimum focal length of the lens')),
        })
      if self.lens.lensmodel.max_focal_length is not None and self.focal_length > self.lens.lensmodel.max_focal_length:
        raise ValidationError({
          'focal_length': ValidationError(('Focal length cannot be longer than the maximum focal length of the lens')),
        })
  def save(self, *args, **kwargs):
    # Auto-populate focal length
    if self.lens:
      if self.lens.lensmodel.zoom is False:
        if self.teleconverter is None:
          self.focal_length = self.lens.lensmodel.min_focal_length
    super().save(*args, **kwargs)

# Table to catalog prints made from negatives
class Print(models.Model):
  negative = models.ForeignKey(Negative, on_delete=models.CASCADE, help_text='Negative that this print was made from')
  date = models.DateField(help_text='The date that the print was made', blank=True, null=True)
  paper_stock = models.ForeignKey(PaperStock, on_delete=models.CASCADE, blank=True, null=True, help_text='Paper stock that this print was made on')
  height = models.DecimalField(help_text='Height of the print in inches', max_digits=4, decimal_places=1, blank=True, null=True)
  width = models.DecimalField(help_text='Width of the print in inches', max_digits=4, decimal_places=1, blank=True, null=True)
  aperture = models.DecimalField(help_text='Aperture used to make this print (numerical part only, e.g. 5.6)', max_digits=3, decimal_places=1, blank=True, null=True)
  exposure_time = models.DurationField(help_text='Exposure time of this print', blank=True, null=True)
  filtration_grade = models.DecimalField(help_text='Contrast grade of paper used', max_digits=2, decimal_places=1, blank=True, null=True, validators=[MinValueValidator(0),MaxValueValidator(5)])
  development_time = models.DurationField(help_text='Development time of this print', blank=True, null=True)
  toner = models.ManyToManyField('Toner', through='Toning', help_text='Toners and bleaches used to treat this print')
  own = models.BooleanField(help_text='Whether we currently own this print', blank=True, null=True)
  location = models.CharField(help_text='The place where this print is currently', max_length=100, blank=True, null=True)
  sold_price = MoneyField(help_text='Sale price of the print', max_digits=12, decimal_places=2, blank=True, null=True, default_currency='GBP')
  enlarger = models.ForeignKey(Enlarger, on_delete=models.CASCADE, blank=True, null=True, help_text='Enlarger used to make this print')
  lens = models.ForeignKey(Lens, on_delete=models.CASCADE, blank=True, null=True, help_text='Enlarger lens used to make this print', limit_choices_to={'lensmodel__mount__purpose': 'Enlarger'})
  developer = models.ForeignKey(Developer, on_delete=models.CASCADE, blank=True, null=True, help_text='Developer used to develop this print', limit_choices_to={'for_paper': True})
  fine = models.BooleanField(help_text='Whether this is a fine print', blank=True, null=True)
  notes = models.CharField(help_text='Freeform notes about this print, e.g. dodging, burning & complex toning', max_length=200, blank=True, null=True)
  archive = models.ForeignKey(Archive, on_delete=models.CASCADE, blank=True, null=True, help_text='Archive that this print is stored in')
  printer = models.ForeignKey(Person, on_delete=models.CASCADE, blank=True, null=True, help_text='Person who made this print')
  owner = CurrentUserField()
  def __str__(self):
    return "#%i" % (self.pk)
  class Meta:
    verbose_name_plural = "prints"
  def clean(self):
    # Aperture must be in range of lens model aperture
    if self.aperture is not None and self.lens is not None:
      if self.lens.lensmodel.max_aperture is not None and self.aperture < self.lens.lensmodel.max_aperture:
        raise ValidationError({
          'aperture': ValidationError(('Aperture cannot be greater than the maximum aperture of the lens')),
        })
      if self.lens.lensmodel.min_aperture is not None and self.aperture > self.lens.lensmodel.min_aperture:
        raise ValidationError({
          'aperture': ValidationError(('Aperture cannot be smaller than the minimum aperture of the lens')),
        })

# Table to track which toners were used on which print
class Toning(models.Model):
  toner = models.ForeignKey(Toner, on_delete=models.CASCADE, help_text='Toner used on this print')
  print = models.ForeignKey(Print, on_delete=models.CASCADE, help_text='Print that was toned')
  dilution = models.CharField(help_text='Dilution of the toner', max_length=8, blank=True, null=True)
  duration = models.DurationField(help_text='Duration of the toning', blank=True, null=True)
  order = models.PositiveIntegerField(help_text='Order in which this toner was applied', blank=True, null=True)
  class Meta:
    ordering = ['order']
    unique_together = ['print', 'order']
  
# Table to catalog all repairs and servicing undertaken on cameras and lenses in the collection
class Repair(models.Model):
  camera = models.ForeignKey(Camera, on_delete=models.CASCADE, blank=True, null=True, help_text='Camera that was repaired')
  lens = models.ForeignKey(Lens, on_delete=models.CASCADE, blank=True, null=True, help_text='Lens that was repaired')
  date = models.DateField(help_text='The date of the repair', blank=True, null=True)
  summary = models.CharField(help_text='Brief summary of the repair', max_length=100)
  description = models.CharField(help_text='Longer description of the repair', max_length=500, blank=True, null=True)
  owner = CurrentUserField()
  class Meta:
    ordering = ['date']
    verbose_name_plural = "repairs"

# Table to record all the images that have been scanned digitally
class Scan(models.Model):
  negative = models.ForeignKey(Negative, on_delete=models.CASCADE, blank=True, null=True, help_text='Negative that this scan was made from')
  print = models.ForeignKey(Print, on_delete=models.CASCADE, blank=True, null=True, help_text='Print that this scan was made from')
  filename = models.CharField(help_text='Filename of the scan', max_length=128)
  date = models.DateField(help_text='Date that this scan was made', blank=True, null=True)
  colour = models.BooleanField(help_text='Whether this is a colour image', blank=True, null=True)
  width = models.PositiveIntegerField(help_text='Width of the scanned image in pixels', blank=True, null=True)
  height = models.PositiveIntegerField(help_text='Height of the scanned image in pixels', blank=True, null=True)
  owner = CurrentUserField()
  def __str__(self):
    return self.filename
  def clean(self):
    # Check print source
    if self.negative is not None and self.print is not None:
      raise ValidationError({
        'negative': ValidationError(('Choose either negative or print')),
        'print': ValidationError(('Choose either negative or print')),
      })
  class Meta:
    verbose_name_plural = "scans"

# Table to record orders for prints
class Order(models.Model):
  negative = models.ForeignKey(Negative, on_delete=models.CASCADE, help_text='Negative that needs to be printed')
  width = models.PositiveIntegerField(help_text='Width of print to be made', blank=True, null=True)
  height = models.PositiveIntegerField(help_text='Height of print to be made', blank=True, null=True)
  added = models.DateField(help_text='Date that the order was placed', blank=True, null=True)
  printed = models.BooleanField(help_text='Whether the print has been made', blank=True, null=True)
  print = models.ForeignKey(Print, on_delete=models.CASCADE, blank=True, null=True, help_text='Print that was made to fulfil this order')
  recipient = models.ForeignKey(Person, on_delete=models.CASCADE, help_text='Person who placed this order')
  owner = CurrentUserField()
  def __str__(self):
    return self.pk
  class Meta:
    ordering = ['added']
    verbose_name_plural = "orders"


from django.contrib import admin

# The text to put at the top of each admin page, as an <h1> (a string). By default, this is “Django administration”.
admin.site.site_header = 'PhotoDB'

# The text to put at the end of each admin page’s <title> (a string). By default, this is “Django site admin”.
admin.site.site_title = 'PhotoDB'

# The text to put at the top of the admin index page (a string). By default, this is “Site administration”.
admin.site.index_title = 'PhotoDB'

# The URL for the “View site” link at the top of each admin page. By default, site_url is /. Set it to None to remove the link.
admin.site.site_url = None

# Import all models that need admin pages
from .models import Accessory, Archive, Battery, BulkFilm, Camera, CameraModel, Developer, Enlarger, FilmStock, Filter
from .models import Flash, FlashProtocol, Format, Lens, LensModel, Manufacturer
from .models import MeteringType, Mount, MountAdapter, NegativeSize, Order, PaperStock, Person, Print, Toning
from .models import Process, Repair, Scan, Negative, Film, Series, ShutterSpeed, Teleconverter, Toner

# Define inlines that can be embedded into other admin pages
class CameraInline(admin.TabularInline):
  model = Camera
  extra = 0

class CameraModelInline(admin.TabularInline):
  model = CameraModel
  extra = 0

class LensInline(admin.TabularInline):
  model = Lens
  extra = 0

class NegativeInline(admin.TabularInline):
  model = Negative
  extra = 0

class PrintInline(admin.TabularInline):
  model = Print
  extra = 0

class ScanInline(admin.TabularInline):
  model = Scan
  extra = 0

class ToningInline(admin.TabularInline):
  model = Print.toner.through
  verbose_name = "toner"
  verbose_name_plural = "toners"
  extra = 0

# Now register the admin pages, customising where necessary
class AccessoryAdmin(admin.ModelAdmin):
  filter_horizontal = ('camera_model_compatibility', 'lens_model_compatibility')
  exclude = ('owner',)
admin.site.register(Accessory, AccessoryAdmin)

class ArchiveAdmin(admin.ModelAdmin):
  exclude = ('owner',)
admin.site.register(Archive, ArchiveAdmin)

admin.site.register(Battery)

class BulkFilmAdmin(admin.ModelAdmin):
  exclude = ('owner',)
admin.site.register(BulkFilm, BulkFilmAdmin)

class CameraAdmin(admin.ModelAdmin):
  exclude = ('owner',)
admin.site.register(Camera, CameraAdmin)

class CameraModelAdmin(admin.ModelAdmin):
  fieldsets = (
    (None, {
      'fields': ('manufacturer', 'model', 'mount', 'format', 'body_type', 'weight', ('introduced', 'discontinued'), 'negative_size', 'cable_release', 'viewfinder_coverage', ('power_drive', 'continuous_fps'), 'fixed_mount', 'lensmodel', ('battery_qty', 'battery_type'), 'notes', 'tripod', 'series'),
      'description': 'Enter information about this camera model',
    }),
    ('Metering', {
      'fields': ('metering', 'metering_type', 'coupled_metering', ('min_iso', 'max_iso'), ('meter_min_ev', 'meter_max_ev'), 'metering_modes', 'exposure_programs')
    }),
    ('Flash', {
      'fields': ('int_flash', 'int_flash_gn', 'ext_flash', 'flash_metering', 'pc_sync', 'hotshoe', 'coldshoe')
    }),
    ('Focus', {
      'fields': ('focus_type', 'af_points', 'dof_preview')
    }),
    ('Shutter', {
      'fields': ('shutter_type', 'shutter_model', 'shutter_speeds', 'bulb', 'time'),
    }),
  )
  inlines = [
    CameraInline,
  ]
  filter_horizontal = ('metering_modes', 'exposure_programs', 'series', 'shutter_speeds')
  search_fields = ['manufacturer__name', 'model', 'notes']
  list_display = ('__str__', 'mount', 'format', 'body_type', 'introduced')
  list_filter = ('manufacturer__name', 'mount', 'format', 'body_type', 'series')
admin.site.register(CameraModel, CameraModelAdmin)

admin.site.register(Developer)

class EnlargerAdmin(admin.ModelAdmin):
  exclude = ('owner',)
admin.site.register(Enlarger, EnlargerAdmin)

admin.site.register(FilmStock)

class FilterAdmin(admin.ModelAdmin):
  exclude = ('owner',)
admin.site.register(Filter, FilterAdmin)

class FlashAdmin(admin.ModelAdmin):
  exclude = ('owner',)
admin.site.register(Flash, FlashAdmin)

admin.site.register(FlashProtocol)

admin.site.register(Format)

class LensAdmin(admin.ModelAdmin):
  exclude = ('owner',)
admin.site.register(Lens, LensAdmin)

class LensModelAdmin(admin.ModelAdmin):
  fieldsets = (
    (None, {
      'fields': ('manufacturer', 'model', 'fixed_mount', 'mount'),
      'description': 'Enter information about this camera model',
    }),
    ('Optics', {
      'fields': ('zoom', ('min_focal_length', 'max_focal_length'), 'closest_focus', ('max_aperture', 'min_aperture'), ('elements', 'groups'), ('nominal_min_angle_diag', 'nominal_max_angle_diag'), 'magnification'),
    }),
    ('Physical', {
      'fields': ('length', 'diameter', 'weight'),
    }),
    ('Other', {
      'fields': ('aperture_blades', 'autofocus', 'filter_thread', 'url', 'introduced', 'discontinued', 'negative_size', 'notes', 'coating', 'hood', 'exif_lenstype', 'rectilinear', 'image_circle', 'formula', 'shutter_model', 'series'),
    })
  )
  inlines = [
    LensInline,
  ]
  filter_horizontal = ('series',)
  search_fields = ['manufacturer__name', 'model', 'notes']
  list_display = ('__str__', 'mount', 'min_focal_length', 'max_aperture', 'introduced')
  list_filter = ('manufacturer__name', 'mount')
admin.site.register(LensModel, LensModelAdmin)

admin.site.register(Manufacturer)

admin.site.register(MeteringType)

admin.site.register(Mount)

class MountAdapterAdmin(admin.ModelAdmin):
  exclude = ('owner',)
admin.site.register(MountAdapter, MountAdapterAdmin)

class NegativeSizeAdmin(admin.ModelAdmin):
  readonly_fields = ('area', 'crop_factor', 'aspect_ratio')
admin.site.register(NegativeSize, NegativeSizeAdmin)

class OrderAdmin(admin.ModelAdmin):
  exclude = ('owner',)
admin.site.register(Order, OrderAdmin)

admin.site.register(PaperStock)

class PersonAdmin(admin.ModelAdmin):
  exclude = ('owner',)
admin.site.register(Person, PersonAdmin)

class PrintAdmin(admin.ModelAdmin):
  fieldsets = (
    (None, {
      'fields': ('negative', 'date', 'notes', 'own', 'location', 'sold_price', 'fine', 'archive', 'printer', ),
    }),
    ('Paper', {
      'fields': ('paper_stock', 'height', 'width', ),
    }),
    ('Exposure', {
      'fields': ('aperture', 'exposure_time', 'filtration_grade', 'enlarger', 'lens',),
    }),
    ('Development', {
      'fields': ('developer', 'development_time',),
    }),
  )
  inlines = [
    ToningInline,
  ]
  exclude = ('owner',)
  search_fields = ['negative__caption', 'notes']
  list_display = ('__str__', 'date', 'negative')
  list_filter = ('paper_stock', 'developer', 'fine')
admin.site.register(Print, PrintAdmin)

admin.site.register(Process)

class RepairAdmin(admin.ModelAdmin):
  exclude = ('owner',)
admin.site.register(Repair, RepairAdmin)

class ScanAdmin(admin.ModelAdmin):
  exclude = ('owner',)
admin.site.register(Scan, ScanAdmin)

class NegativeAdmin(admin.ModelAdmin):
  fieldsets = (
    (None, {
      'fields': (('film', 'frame'), 'caption', 'date', 'notes', 'photographer'),
      'description': 'Enter information about this camera model',
    }),
    ('Exposure', {
      'fields': ('lens', 'mount_adapter', 'shutter_speed', 'aperture', 'filter', 'teleconverter', 'focal_length', 'flash', 'metering_mode', 'exposure_program', 'copy_of'),
    }),
    ('Location', {
      'fields': (('latitude', 'longitude'),),
    }),
  )
  search_fields = ['caption', 'notes']
  list_display = ('__str__', 'caption', 'date',)
  list_filter = ('film', 'photographer')
  inlines = [
    ScanInline,
    PrintInline,
  ]
  exclude = ('owner',)
admin.site.register(Negative, NegativeAdmin)

class FilmAdmin(admin.ModelAdmin):
  fieldsets = (
    (None, {
      'fields': ('format', 'title', 'filmstock', 'film_batch', 'expiry_date', 'purchase_date', 'price', 'frames'),
    }),
    ('Bulk film', {
      'fields': ('bulk_film', 'bulk_film_loaded'),
    }),
    ('Exposure', {
      'fields': ('exposed_at', 'camera', 'date_loaded'),
    }),
    ('Development', {
      'fields': ('developer', 'dev_uses', 'dev_time', 'dev_temp', 'dev_n', 'development_notes', 'processed_by', 'date_processed'),
    }),
    ('Archive', {
      'fields': ('directory', 'archive'),
    })
  )
  inlines = [
    NegativeInline,
  ]
  exclude = ('owner',)
admin.site.register(Film, FilmAdmin)

class SeriesAdmin(admin.ModelAdmin):
  exclude = ('owner',)
admin.site.register(Series, SeriesAdmin)

class ShutterSpeedAdmin(admin.ModelAdmin):
  readonly_fields = ('duration',)
admin.site.register(ShutterSpeed, ShutterSpeedAdmin)

class TeleconverterAdmin(admin.ModelAdmin):
  exclude = ('owner',)
admin.site.register(Teleconverter, TeleconverterAdmin)

admin.site.register(Toner)


from django.db import migrations

def forwards_func(apps, schema_editor):
  # We get the model from the versioned app registry;
  # if we directly import it, it'll be the wrong version
  db_alias = schema_editor.connection.alias

  Filmstock = apps.get_model("schema", "Filmstock")
  Manufacturer = apps.get_model("schema", "Manufacturer")
  Process = apps.get_model("schema", "Process")

  # Load processes
  BW = Process.objects.get(name='Black & white negative')
  C41 = Process.objects.get(name='C-41')
  E6 = Process.objects.get(name='E-6')
  Polaroid = Process.objects.get(name='Polaroid')
  K14 = Process.objects.get(name='K-14')

  Agfa = Manufacturer.objects.get(name='Agfa')
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Agfa, name="Vista Plus 200", iso=200, colour=True, process=C41, panchromatic=True)
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Agfa, name="HDC Plus", iso=200, colour=True, process=C41, panchromatic=True)

  Boots = Manufacturer.objects.get(name='Boots')
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Boots, name="200", iso=200, colour=True, process=C41, panchromatic=True)
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Boots, name="200 Colour Slide", iso=200, colour=True, process=E6, panchromatic=True)

  Efke = Manufacturer.objects.get(name='Efke')
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Efke, name="R50", iso=50, colour=False, process=BW, panchromatic=True)
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Efke, name="R25", iso=25, colour=False, process=BW, panchromatic=True)
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Efke, name="IR820", iso=100, colour=False, process=BW, panchromatic=True)
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Efke, name="IR820 Aura", iso=100, colour=False, process=BW, panchromatic=True)

  Fuji = Manufacturer.objects.get(name='Fuji')
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Fuji, name="Reala 100", iso=100, colour=True, process=C41, panchromatic=True)
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Fuji, name="Acros 100", iso=100, colour=False, process=BW, panchromatic=True)
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Fuji, name="Neopan 1600", iso=1600, colour=False, process=BW, panchromatic=True)
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Fuji, name="S 400", iso=400, colour=True, process=C41, panchromatic=True)
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Fuji, name="S 200", iso=200, colour=True, process=C41, panchromatic=True)
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Fuji, name="Pro 160S", iso=160, colour=True, process=C41, panchromatic=True)
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Fuji, name="Neopan 400", iso=400, colour=False, process=BW, panchromatic=True)
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Fuji, name="Sensia 200", iso=200, colour=True, process=E6, panchromatic=True)
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Fuji, name="Superia 200", iso=200, colour=True, process=C41, panchromatic=True)
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Fuji, name="Velvia 50", iso=50, colour=True, process=E6, panchromatic=True)
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Fuji, name="Velvia 100F", iso=100, colour=True, process=E6, panchromatic=True)
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Fuji, name="Superia X-Tra", iso=400, colour=True, process=C41, panchromatic=True)
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Fuji, name="Superia 100", iso=100, colour=True, process=C41, panchromatic=True)
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Fuji, name="Unknown Slide", colour=True, panchromatic=True)

  Ilford = Manufacturer.objects.get(name='Ilford')
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Ilford, name="FP4+ 125", iso=125, colour=False, process=BW, panchromatic=True)
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Ilford, name="HP5+ 400", iso=400, colour=False, process=BW, panchromatic=True)
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Ilford, name="Delta 100", iso=100, colour=False, process=BW, panchromatic=True)
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Ilford, name="Delta 400", iso=400, colour=False, process=BW, panchromatic=True)
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Ilford, name="FP3", colour=False, process=BW, panchromatic=True)
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Ilford, name="Pan F+ 50", iso=50, colour=False, process=BW, panchromatic=True)
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Ilford, name="Pan 100", iso=100, colour=False, process=BW, panchromatic=True)
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Ilford, name="Delta 3200", iso=3200, colour=False, process=BW, panchromatic=True)
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Ilford, name="FP4", iso=125, colour=False, process=BW, panchromatic=True)

  Jessops = Manufacturer.objects.get(name='Jessops')
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Jessops, name="Pan 100S", iso=100, colour=False, process=BW, panchromatic=True)

  KawausoShoten = Manufacturer.objects.get(name='Kawauso-Shoten')
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=KawausoShoten, name="Rera Pan", iso=100, colour=False, process=BW, panchromatic=True)

  Kentmere = Manufacturer.objects.get(name='Kentmere')
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Kentmere, name="100", iso=100, colour=False, process=BW, panchromatic=True)
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Kentmere, name="Select RC", iso=6, colour=False, process=BW, panchromatic=False)

  Kodak = Manufacturer.objects.get(name='Kodak')
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Kodak, name="Gold", iso=400, colour=True, process=C41, panchromatic=True)
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Kodak, name="GT 800", iso=800, colour=True, process=C41, panchromatic=True)
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Kodak, name="200", iso=200, colour=True, process=C41, panchromatic=True)
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Kodak, name="VR Plus 400", iso=400, colour=True, process=C41, panchromatic=True)
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Kodak, name="Ektar 100", iso=100, colour=True, process=C41, panchromatic=True)
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Kodak, name="Kodacolor 200", iso=200, colour=True, process=C41, panchromatic=True)
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Kodak, name="Elitechrome 100", iso=100, colour=True, process=E6, panchromatic=True)
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Kodak, name="Ektar 125", iso=125, colour=True, process=C41, panchromatic=True)
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Kodak, name="Gold 200", iso=200, colour=True, process=C41, panchromatic=True)
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Kodak, name="Kodachrome 40", iso=40, colour=True, process=K14, panchromatic=True)
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Kodak, name="Kodalith", iso=12, colour=False, process=BW, panchromatic=False)
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Kodak, name="Ektachrome 50 Tungsten", iso=50, colour=True, process=E6, panchromatic=True)
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Kodak, name="Plus-X pan", iso=125, colour=False, process=BW, panchromatic=True)
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Kodak, name="Electron Image Film SO-163", iso=12, colour=False, process=BW, panchromatic=False)
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Kodak, name="High-Speed Infrared HIE", colour=False, process=BW, panchromatic=True)
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Kodak, name="VR Plus 200", iso=200, colour=True, process=C41, panchromatic=True)
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Kodak, name="Kodachrome 10", iso=10, colour=True, process=K14, panchromatic=True)

  Konica = Manufacturer.objects.get(name='Konica')
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Konica, name="VX 200", iso=200, colour=True, process=C41, panchromatic=True)

  Maco = Manufacturer.objects.get(name='Maco')
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Maco, name="IR820", iso=400, colour=False, process=BW, panchromatic=True)

  ORWO = Manufacturer.objects.get(name='ORWO')
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=ORWO, name="ORWOChrom UK17", iso=40, colour=True, process=E6, panchromatic=True)

  TheImpossibleProject = Manufacturer.objects.get(name='The Impossible Project')
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=TheImpossibleProject, name="PX 600 Silver Shade", iso=600, colour=False, process=Polaroid, panchromatic=True)

  Tudor = Manufacturer.objects.get(name='Tudor')
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Tudor, name="Colour 200", iso=200, colour=True, process=C41, panchromatic=True)
  Filmstock.objects.using(db_alias).update_or_create(manufacturer=Tudor, name="XLX 100", iso=100, colour=True, process=C41, panchromatic=True)

class Migration(migrations.Migration):

    dependencies = [
        ('schema', '0007_mounts'),
    ]

    operations = [
        migrations.RunPython(forwards_func),
    ]



from django.db import migrations

def forwards_func(apps, schema_editor):
  # We get the model from the versioned app registry;
  # if we directly import it, it'll be the wrong version
  db_alias = schema_editor.connection.alias

  NegativeSize = apps.get_model("schema", "NegativeSize")
  NegativeSize.objects.using(db_alias).bulk_create([
    NegativeSize(name="6x7", width="70", height="56", crop_factor="0.48", area="3920", aspect_ratio="1.25"),
    NegativeSize(name="6x6", width="56", height="56", crop_factor="0.55", area="3136", aspect_ratio="1"),
    NegativeSize(name="6x4.5", width="56", height="41.5", crop_factor="0.62", area="2324", aspect_ratio="1.35"),
    NegativeSize(name="6x9", width="84", height="56", crop_factor="0.43", area="4704", aspect_ratio="1.5"),
    NegativeSize(name="35mm", width="36", height="24", crop_factor="1", area="864", aspect_ratio="1.5"),
    NegativeSize(name="APS-C", width="22.2", height="14.8", crop_factor="1.62", area="329", aspect_ratio="1.5"),
    NegativeSize(name="35mm half frame", width="18", height="24", crop_factor="1.44", area="432", aspect_ratio="0.75"),
    NegativeSize(name="1/1.6\"", width="8", height="6", crop_factor="4.33", area="48", aspect_ratio="1.33"),
    NegativeSize(name="Quarter plate", width="110", height="82.6", crop_factor="0.31", area="9086", aspect_ratio="1.33"),
    NegativeSize(name="Integral 600", width="79", height="79", crop_factor="0.39", area="6241", aspect_ratio="1"),
    NegativeSize(name="35mm panoramic", width="36", height="12", crop_factor="1.14", area="432", aspect_ratio="3"),
    NegativeSize(name="6.5x9", width="88.9", height="63.5", crop_factor="0.4", area="5645", aspect_ratio="1.4"),
    NegativeSize(name="2.25x3.25\"", width="82", height="57", crop_factor="0.43", area="4674", aspect_ratio="1.44"),
    NegativeSize(name="4x5", width="125", height="100", crop_factor="0.27", area="12500", aspect_ratio="1.25"),
    NegativeSize(name="8x10", width="238", height="190", crop_factor="0.14", area="45220", aspect_ratio="1.25"),
    NegativeSize(name="126", width="28", height="28", crop_factor="1.09", area="784", aspect_ratio="1"),
    NegativeSize(name="110", width="17", height="13", crop_factor="2.02", area="221", aspect_ratio="1.31"),
    NegativeSize(name="1\"x1\"", width="24", height="24", crop_factor="1.27", area="576", aspect_ratio="1"),
    NegativeSize(name="2.5\" x 4.25\"", width="108", height="63.5", crop_factor="0.35", area="6658", aspect_ratio="1.7"),
    NegativeSize(name="6x11", width="108", height="55", crop_factor="0.36", area="5940", aspect_ratio="1.96"),
    NegativeSize(name="1?x2½\"", width="60", height="40", crop_factor="0.6", area="2400", aspect_ratio="1.5"),
  ])


class Migration(migrations.Migration):

    dependencies = [
        ('schema', '0005_formats'),
    ]

    operations = [
        migrations.RunPython(forwards_func),
    ]



from django.db import migrations

def forwards_func(apps, schema_editor):
  # We get the model from the versioned app registry;
  # if we directly import it, it'll be the wrong version
  db_alias = schema_editor.connection.alias

  Format = apps.get_model("schema", "Format")
  Format.objects.using(db_alias).bulk_create([
    Format(format="120 roll"),
    Format(format="135 roll"),
    Format(format="620 roll"),
    Format(format="APS-C"),
    Format(format="Quarter plate"),
    Format(format="Polaroid 600"),
    Format(format="2.25x3.25\" sheet"),
    Format(format="6.5x9 sheet"),
    Format(format="4x5\" sheet"),
    Format(format="126 cartridge"),
    Format(format="110 cartridge"),
    Format(format="3¼x4\""),
    Format(format="116 roll"),
    Format(format="127 roll"),
  ])


class Migration(migrations.Migration):

    dependencies = [
        ('schema', '0004_manufacturers'),
    ]

    operations = [
        migrations.RunPython(forwards_func),
    ]



from django.db import migrations

def forwards_func(apps, schema_editor):
  # We get the model from the versioned app registry;
  # if we directly import it, it'll be the wrong version
  db_alias = schema_editor.connection.alias

  ShutterSpeed = apps.get_model("schema", "ShutterSpeed")
  ShutterSpeed.objects.using(db_alias).bulk_create([
    ShutterSpeed(shutter_speed="1/8000", duration="0.000125"),
    ShutterSpeed(shutter_speed="1/6400", duration="0.00015625"),
    ShutterSpeed(shutter_speed="1/6000", duration="0.000166666666667"),
    ShutterSpeed(shutter_speed="1/5000", duration="0.0002"),
    ShutterSpeed(shutter_speed="1/4000", duration="0.00025"),
    ShutterSpeed(shutter_speed="1/3200", duration="0.0003125"),
    ShutterSpeed(shutter_speed="1/3000", duration="0.000333333333333"),
    ShutterSpeed(shutter_speed="1/2500", duration="0.0004"),
    ShutterSpeed(shutter_speed="1/2000", duration="0.0005"),
    ShutterSpeed(shutter_speed="1/1600", duration="0.000625"),
    ShutterSpeed(shutter_speed="1/1500", duration="0.000666666666667"),
    ShutterSpeed(shutter_speed="1/1250", duration="0.0008"),
    ShutterSpeed(shutter_speed="1/1000", duration="0.001"),
    ShutterSpeed(shutter_speed="1/800", duration="0.00125"),
    ShutterSpeed(shutter_speed="1/750", duration="0.001333333333333"),
    ShutterSpeed(shutter_speed="1/640", duration="0.0015625"),
    ShutterSpeed(shutter_speed="1/500", duration="0.002"),
    ShutterSpeed(shutter_speed="1/400", duration="0.0025"),
    ShutterSpeed(shutter_speed="1/350", duration="0.002857142857143"),
    ShutterSpeed(shutter_speed="1/320", duration="0.003125"),
    ShutterSpeed(shutter_speed="1/250", duration="0.004"),
    ShutterSpeed(shutter_speed="1/200", duration="0.005"),
    ShutterSpeed(shutter_speed="1/180", duration="0.005555555555556"),
    ShutterSpeed(shutter_speed="1/160", duration="0.00625"),
    ShutterSpeed(shutter_speed="1/125", duration="0.008"),
    ShutterSpeed(shutter_speed="1/100", duration="0.01"),
    ShutterSpeed(shutter_speed="1/90", duration="0.011111111111111"),
    ShutterSpeed(shutter_speed="1/80", duration="0.0125"),
    ShutterSpeed(shutter_speed="1/60", duration="0.016666666666667"),
    ShutterSpeed(shutter_speed="1/50", duration="0.02"),
    ShutterSpeed(shutter_speed="1/45", duration="0.022222222222222"),
    ShutterSpeed(shutter_speed="1/40", duration="0.025"),
    ShutterSpeed(shutter_speed="1/30", duration="0.033333333333333"),
    ShutterSpeed(shutter_speed="1/25", duration="0.04"),
    ShutterSpeed(shutter_speed="1/20", duration="0.05"),
    ShutterSpeed(shutter_speed="1/15", duration="0.066666666666667"),
    ShutterSpeed(shutter_speed="1/13", duration="0.076923076923077"),
    ShutterSpeed(shutter_speed="1/10", duration="0.1"),
    ShutterSpeed(shutter_speed="1/8", duration="0.125"),
    ShutterSpeed(shutter_speed="1/6", duration="0.166666666666667"),
    ShutterSpeed(shutter_speed="1/5", duration="0.2"),
    ShutterSpeed(shutter_speed="1/4", duration="0.25"),
    ShutterSpeed(shutter_speed="1/3", duration="0.333333333333333"),
    ShutterSpeed(shutter_speed="1/2.5", duration="0.4"),
    ShutterSpeed(shutter_speed="1/2", duration="0.5"),
    ShutterSpeed(shutter_speed="1/1.5", duration="0.666666666666667"),
    ShutterSpeed(shutter_speed="1", duration="1"),
    ShutterSpeed(shutter_speed="1.3", duration="1.3"),
    ShutterSpeed(shutter_speed="1.5", duration="1.5"),
    ShutterSpeed(shutter_speed="1.6", duration="1.6"),
    ShutterSpeed(shutter_speed="1/0.6", duration="1.66666666666667"),
    ShutterSpeed(shutter_speed="2", duration="2"),
    ShutterSpeed(shutter_speed="2.5", duration="2.5"),
    ShutterSpeed(shutter_speed="3", duration="3"),
    ShutterSpeed(shutter_speed="1/0.3", duration="3.33333333333333"),
    ShutterSpeed(shutter_speed="4", duration="4"),
    ShutterSpeed(shutter_speed="5", duration="5"),
    ShutterSpeed(shutter_speed="6", duration="6"),
    ShutterSpeed(shutter_speed="8", duration="8"),
    ShutterSpeed(shutter_speed="10", duration="10"),
    ShutterSpeed(shutter_speed="12", duration="12"),
    ShutterSpeed(shutter_speed="13", duration="13"),
    ShutterSpeed(shutter_speed="16", duration="16"),
    ShutterSpeed(shutter_speed="20", duration="20"),
    ShutterSpeed(shutter_speed="24", duration="24"),
    ShutterSpeed(shutter_speed="26", duration="26"),
    ShutterSpeed(shutter_speed="30", duration="30")
  ]) 

class Migration(migrations.Migration):

    dependencies = [
        ('schema', '0002_data'),
    ]

    operations = [
        migrations.RunPython(forwards_func),
    ]



# Generated by Django 2.2.4 on 2019-10-16 14:03

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('schema', '0012_auto_20191016_1106'),
    ]

    operations = [
        migrations.DeleteModel(
            name='FilterAdapter',
        ),
    ]



from django.db import migrations

def forwards_func(apps, schema_editor):
  # We get the model from the versioned app registry;
  # if we directly import it, it'll be the wrong version
  db_alias = schema_editor.connection.alias

  Mount = apps.get_model("schema", "Mount")
  Manufacturer = apps.get_model("schema", "Manufacturer")
  
  Manufacturer.objects.using(db_alias).bulk_create([
    Manufacturer(name="Alpa", country="Switzerland", url="http://www.alpa.ch"),
    Manufacturer(name="Bronica", city="Tokyo", country="Japan", founded="1956"),
    Manufacturer(name="Hasselblad", city="Gothenburg", country="Sweden", url="https://www.hasselblad.com", founded="1841"),
    Manufacturer(name="Ihagee", city="Dresden", country="Germany", founded="1912"),
    Manufacturer(name="Kowa", country="Japan", founded="1954"),
    Manufacturer(name="Petri", country="Japan", founded="1907"),
  ], ignore_conflicts=True)

  Alpa = Manufacturer.objects.get(name='Alpa')
  Mount.objects.using(db_alias).update_or_create(mount="Alpa", type='Bayonet', purpose='Camera', manufacturer=Alpa)

  Braun = Manufacturer.objects.get(name='Braun')
  Mount.objects.using(db_alias).update_or_create(mount="Paxette", shutter_in_lens=0, type='Screw', purpose='Camera', manufacturer=Braun)

  Bronica = Manufacturer.objects.get(name='Bronica')
  Mount.objects.using(db_alias).update_or_create(mount="Bronica ETR", type='Bayonet', purpose='Camera', manufacturer=Bronica)
  Mount.objects.using(db_alias).update_or_create(mount="Bronica GS1", type='Bayonet', purpose='Camera', manufacturer=Bronica)
  Mount.objects.using(db_alias).update_or_create(mount="Bronica RF", type='Bayonet', purpose='Camera', manufacturer=Bronica)
  Mount.objects.using(db_alias).update_or_create(mount="Bronica SQA", type='Bayonet', purpose='Camera', manufacturer=Bronica)

  Canon = Manufacturer.objects.get(name='Canon')
  Mount.objects.using(db_alias).update_or_create(mount="Canon EF", shutter_in_lens=0, type='Bayonet', purpose='Camera', manufacturer=Canon)
  Mount.objects.using(db_alias).update_or_create(mount="Canon EF-M", shutter_in_lens=0, type='Bayonet', purpose='Camera', manufacturer=Canon)
  Mount.objects.using(db_alias).update_or_create(mount="Canon EF-S", shutter_in_lens=0, type='Bayonet', purpose='Camera', manufacturer=Canon)
  Mount.objects.using(db_alias).update_or_create(mount="Canon EX", shutter_in_lens=0, type='Bayonet', purpose='Camera', manufacturer=Canon)
  Mount.objects.using(db_alias).update_or_create(mount="Canon FD", shutter_in_lens=0, type='Breech lock', purpose='Camera', manufacturer=Canon)
  Mount.objects.using(db_alias).update_or_create(mount="Canon FL", shutter_in_lens=0, type='Breech lock', purpose='Camera', manufacturer=Canon)

  Fuji = Manufacturer.objects.get(name='Fuji')
  Mount.objects.using(db_alias).update_or_create(mount="Fujica X", type='Bayonet', purpose='Camera', manufacturer=Fuji)

  Hasselblad = Manufacturer.objects.get(name='Hasselblad')
  Mount.objects.using(db_alias).update_or_create(mount="Hasselblad", type='Bayonet', purpose='Camera', manufacturer=Hasselblad)
  Mount.objects.using(db_alias).update_or_create(mount="Hasselblad Xpan", type='Bayonet', purpose='Camera', manufacturer=Hasselblad)

  Ihagee = Manufacturer.objects.get(name='Ihagee')
  Mount.objects.using(db_alias).update_or_create(mount="Exakta", type='Bayonet', purpose='Camera', manufacturer=Ihagee)

  Konica = Manufacturer.objects.get(name='Konica')
  Mount.objects.using(db_alias).update_or_create(mount="KM", type='Bayonet', purpose='Camera', manufacturer=Konica)
  Mount.objects.using(db_alias).update_or_create(mount="Konica AR", type='Bayonet', purpose='Camera', manufacturer=Konica)
  Mount.objects.using(db_alias).update_or_create(mount="Konica F", type='Bayonet', purpose='Camera', manufacturer=Konica)

  Kowa = Manufacturer.objects.get(name='Kowa')
  Mount.objects.using(db_alias).update_or_create(mount="Kowa Six", type='Breech lock', purpose='Camera', manufacturer=Kowa)

  Leitz = Manufacturer.objects.get(name='Leitz')
  Mount.objects.using(db_alias).update_or_create(mount="Leica M", shutter_in_lens=0, type='Bayonet', purpose='Camera', manufacturer=Leitz)
  Mount.objects.using(db_alias).update_or_create(mount="Leica R", shutter_in_lens=0, type='Bayonet', purpose='Camera', manufacturer=Leitz)

  Mamiya = Manufacturer.objects.get(name='Mamiya')
  Mount.objects.using(db_alias).update_or_create(mount="Mamiya 6", type='Bayonet', purpose='Camera', manufacturer=Mamiya)
  Mount.objects.using(db_alias).update_or_create(mount="Mamiya 645", type='Bayonet', purpose='Camera', manufacturer=Mamiya)
  Mount.objects.using(db_alias).update_or_create(mount="Mamiya 7/7II", type='Bayonet', purpose='Camera', manufacturer=Mamiya)
  Mount.objects.using(db_alias).update_or_create(mount="Mamiya RB67", shutter_in_lens=1, type='Bayonet', purpose='Camera', manufacturer=Mamiya)
  Mount.objects.using(db_alias).update_or_create(mount="Mamiya RZ67", shutter_in_lens=1, type='Bayonet', purpose='Camera', manufacturer=Mamiya)
  Mount.objects.using(db_alias).update_or_create(mount="Mamiya ZE", shutter_in_lens=0, type='Bayonet', purpose='Camera', manufacturer=Mamiya)
  Mount.objects.using(db_alias).update_or_create(mount="Mamiya/Sekor E", shutter_in_lens=0, type='Bayonet', purpose='Camera', manufacturer=Mamiya)

  Minolta = Manufacturer.objects.get(name='Minolta')
  Mount.objects.using(db_alias).update_or_create(mount="Minolta A", type='Bayonet', purpose='Camera', manufacturer=Minolta)
  Mount.objects.using(db_alias).update_or_create(mount="Minolta SR", type='Bayonet', purpose='Camera', manufacturer=Minolta)
  Mount.objects.using(db_alias).update_or_create(mount="Miranda M44", type='Bayonet', purpose='Camera', manufacturer=Minolta)

  Nikon = Manufacturer.objects.get(name='Nikon')
  Mount.objects.using(db_alias).update_or_create(mount="Nikon F", shutter_in_lens=0, type='Bayonet', purpose='Camera', manufacturer=Nikon)
  Mount.objects.using(db_alias).update_or_create(mount="Nikon S", shutter_in_lens=0, type='Bayonet', purpose='Camera', manufacturer=Nikon)

  Olympus = Manufacturer.objects.get(name='Olympus')
  Mount.objects.using(db_alias).update_or_create(mount="Olympus OM", shutter_in_lens=0, type='Bayonet', purpose='Camera', manufacturer=Olympus)
  Mount.objects.using(db_alias).update_or_create(mount="Olympus Pen F", type='Bayonet', purpose='Camera', manufacturer=Olympus)

  Pentacon = Manufacturer.objects.get(name='Pentacon')
  Mount.objects.using(db_alias).update_or_create(mount="Pentacon Six", type='Breech lock', purpose='Camera', manufacturer=Pentacon)
  Mount.objects.using(db_alias).update_or_create(mount="Praktica", type='Bayonet', purpose='Camera', manufacturer=Pentacon)
  Mount.objects.using(db_alias).update_or_create(mount="Praktina", type='Breech lock', purpose='Camera', manufacturer=Pentacon)

  Pentax = Manufacturer.objects.get(name='Pentax')
  Mount.objects.using(db_alias).update_or_create(mount="Pentax 110", type='Bayonet', purpose='Camera', manufacturer=Pentax)
  Mount.objects.using(db_alias).update_or_create(mount="Pentax 645", type='Bayonet', purpose='Camera', manufacturer=Pentax)
  Mount.objects.using(db_alias).update_or_create(mount="Pentax 6x7", type='Bayonet', purpose='Camera', manufacturer=Pentax)
  Mount.objects.using(db_alias).update_or_create(mount="Pentax K", type='Bayonet', purpose='Camera', manufacturer=Pentax)

  Petri = Manufacturer.objects.get(name='Petri')
  Mount.objects.using(db_alias).update_or_create(mount="Petriflex", type='Breech lock', purpose='Camera', manufacturer=Petri)

  Rollei = Manufacturer.objects.get(name='Rollei')
  Mount.objects.using(db_alias).update_or_create(mount="Rolleiflex SL35", type='Bayonet', purpose='Camera', manufacturer=Rollei)
  Mount.objects.using(db_alias).update_or_create(mount="Rolleiflex SL66", type='Bayonet', purpose='Camera', manufacturer=Rollei)

  Sigma = Manufacturer.objects.get(name='Sigma')
  Mount.objects.using(db_alias).update_or_create(mount="Sigma SA", shutter_in_lens=0, type='Bayonet', purpose='Camera', manufacturer=Sigma)

  Tamron = Manufacturer.objects.get(name='Tamron')
  Mount.objects.using(db_alias).update_or_create(mount="Adaptall", shutter_in_lens=0, type='Bayonet', purpose='Camera', manufacturer=Tamron)

  Zeiss = Manufacturer.objects.get(name='Zeiss Ikon')
  Mount.objects.using(db_alias).update_or_create(mount="Contax G", type='Breech lock', purpose='Camera', manufacturer=Zeiss)
  Mount.objects.using(db_alias).update_or_create(mount="Contax N", type='Bayonet', purpose='Camera', manufacturer=Zeiss)
  Mount.objects.using(db_alias).update_or_create(mount="Contax RF", type='Bayonet', purpose='Camera', manufacturer=Zeiss)
  Mount.objects.using(db_alias).update_or_create(mount="Contax/Yashica", type='Bayonet', purpose='Camera', manufacturer=Zeiss)

  Mount.objects.using(db_alias).update_or_create(mount="C", shutter_in_lens=0, type='Screw', purpose='Camera')
  Mount.objects.using(db_alias).update_or_create(mount="M25 Biological", type='Screw', purpose='Camera')
  Mount.objects.using(db_alias).update_or_create(mount="M37", type='Screw', purpose='Camera')
  Mount.objects.using(db_alias).update_or_create(mount="M39 (Enlarger)", type='Screw', purpose='Enlarger')
  Mount.objects.using(db_alias).update_or_create(mount="M39 (Rangefinder)", shutter_in_lens=0, type='Screw', purpose='Camera')
  Mount.objects.using(db_alias).update_or_create(mount="M39 (SLR)", shutter_in_lens=0, type='Screw', purpose='Camera')
  Mount.objects.using(db_alias).update_or_create(mount="M42", shutter_in_lens=0, type='Screw', purpose='Camera')
  Mount.objects.using(db_alias).update_or_create(mount="Praktiflex", type='Screw', purpose='Camera')
  Mount.objects.using(db_alias).update_or_create(mount="T/T2", shutter_in_lens=0, type='Screw', purpose='Camera')

class Migration(migrations.Migration):

    dependencies = [
        ('schema', '0006_negative_sizes'),
    ]

    operations = [
        migrations.RunPython(forwards_func),
    ]



# Generated by Django 2.1.10 on 2019-08-13 14:11

from django.db import migrations

def forwards_func(apps, schema_editor):
  # We get the model from the versioned app registry;
  # if we directly import it, it'll be the wrong version
  db_alias = schema_editor.connection.alias

  Condition = apps.get_model("schema", "Condition")
  Condition.objects.using(db_alias).bulk_create([
      Condition(code="NEW", name="Brand new", min_rating="100", max_rating="100", description="Never used and still sealed. All original packaging, manuals and accessories included."),
      Condition(code="MINT", name="Like new", min_rating="97", max_rating="99", description="Basically new with very little use.  All boxes and original packaging"),
      Condition(code="EXC+", name="Excellent Plus", min_rating="90", max_rating="96", description="Lens Glass very clean – cosmetically may show very slight wear and/or signs of use but only under very close inspection."),
      Condition(code="EXC", name="Excellent", min_rating="84", max_rating="89", description="Shows slight signs of use – Lens Glass is very clean but may have some dust which will not affect picture quality."),
      Condition(code="GOOD", name="Good", min_rating="75", max_rating="83", description="Appears well used and may include dings, brassing, scrapes and bruises but is in fully functional condition. Glass may have cleaning marks or small scratches which won’t affect picture quality."),
      Condition(code="FAIR", name="Fair", min_rating="60", max_rating="74", description="Appears to have been used very heavily with multiple dings, scrapes, scratches and heavy brassing. Lens Glass may have slight fungus, excessive dust and/or scratches that will likely affect picture quality."),
      Condition(code="POOR", name="Poor", min_rating="50", max_rating="59", description="Very rough looking. Multiple impressions metal, extreme finish loss and excessive brassing. Glass will have marks, fungus and/or haze which will affect picture quality"),
      Condition(code="BROKEN", name="Broken", min_rating="0", max_rating="49", description="Has one or more faults")
  ])

  ExposureProgram = apps.get_model("schema", "ExposureProgram")
  ExposureProgram.objects.using(db_alias).bulk_create([
    ExposureProgram(name="Fixed"),
    ExposureProgram(name="Manual"),
    ExposureProgram(name="Program AE"),
    ExposureProgram(name="Aperture-priority AE"),
    ExposureProgram(name="Shutter speed priority AE"),
    ExposureProgram(name="Creative (Slow speed)"),
    ExposureProgram(name="Action (High speed)"),
    ExposureProgram(name="Portrait"),
    ExposureProgram(name="Landscape"),
    ExposureProgram(name="Bulb"),
  ])

  MeteringMode = apps.get_model("schema", "MeteringMode")
  MeteringMode.objects.using(db_alias).bulk_create([
    MeteringMode(name="None"),
    MeteringMode(name="Average"),
    MeteringMode(name="Center-weighted Average"),
    MeteringMode(name="Spot"),
    MeteringMode(name="Multi-spot"),
    MeteringMode(name="Multi-segment"),
    MeteringMode(name="Partial"),
  ])

  MeteringType = apps.get_model("schema", "MeteringType")
  MeteringType.objects.using(db_alias).bulk_create([
    MeteringType(name="Cadmium sulphide (CdS)"),
    MeteringType(name="Selenium"),
    MeteringType(name="Silicon"),
  ])

  Process = apps.get_model("schema", "Process")
  Process.objects.using(db_alias).bulk_create([
    Process(name="Black & white negative", colour=0, positive=0),
    Process(name="Black & white reversal", colour=0, positive=1),
    Process(name="C-41", colour=1, positive=0),
    Process(name="E-6", colour=1, positive=1),
    Process(name="Polaroid", colour=1, positive=1),
    Process(name="K-14", colour=1, positive=1),
    Process(name="Cyanotype", colour=0, positive=0),
  ])

class Migration(migrations.Migration):

    dependencies = [
        ('schema', '0001_initial'),
    ]

    operations = [
        migrations.RunPython(forwards_func),
    ]



"""
Django settings for photodb project.

Generated by 'django-admin startproject' using Django 2.1.10.

For more information on this file, see
https://docs.djangoproject.com/en/2.1/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/2.1/ref/settings/
"""

import os
from getenv import env

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/2.1/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = env('SECRET_KEY', 'OverrideMe!'),

# SECURITY WARNING: don't run with debug turned on in production!
if os.getenv('DJANGO_ENV') == 'prod':
    DEBUG = False
    ALLOWED_HOSTS = ['*']
else:
    DEBUG = True
    ALLOWED_HOSTS = []

# Application definition

INSTALLED_APPS = [
    'fluent_dashboard',
    'admin_tools',
    'admin_tools.theming',
    'admin_tools.menu',
    'admin_tools.dashboard',
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'schema.apps.SchemaConfig',
    'djmoney',
    'favicon',
    'django_createsuperuserwithpassword',
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'django_currentuser.middleware.ThreadLocalUserMiddleware',
]

ROOT_URLCONF = 'photodb.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
            'loaders': [
                'django.template.loaders.filesystem.Loader',
                'django.template.loaders.app_directories.Loader',
                'admin_tools.template_loaders.Loader',
            ],
        },
    },
]

WSGI_APPLICATION = 'photodb.wsgi.application'


# Database
# https://docs.djangoproject.com/en/2.1/ref/settings/#databases
# Configure databases by setting env vars DB_*

DATABASES = {
    'default': {
        'ENGINE': env('DB_ENGINE', 'django.db.backends.sqlite3'),
        'NAME': env('DB_NAME', os.path.join(BASE_DIR, 'db', 'db.sqlite3')),
        'USER': env('DB_USER'),
        'PASSWORD': env('DB_PASS'),
        'HOST': env('DB_HOST'),
        'PORT': env('DB_PORT'),
    }
}

# Password validation
# https://docs.djangoproject.com/en/2.1/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]


# Internationalization
# https://docs.djangoproject.com/en/2.1/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/2.1/howto/static-files/

STATIC_ROOT = os.path.join(BASE_DIR, "static")
STATIC_URL = '/static/'

# django-fluent-dashboard
ADMIN_TOOLS_INDEX_DASHBOARD = 'fluent_dashboard.dashboard.FluentIndexDashboard'
ADMIN_TOOLS_APP_INDEX_DASHBOARD = 'fluent_dashboard.dashboard.FluentAppIndexDashboard'
ADMIN_TOOLS_MENU = 'fluent_dashboard.menu.FluentMenu'

FLUENT_DASHBOARD_DEFAULT_ICON = 'icons/unknown.png'
FLUENT_DASHBOARD_DEFAULT_MODULE = 'admin_tools.dashboard.modules.AppList'

FLUENT_DASHBOARD_APP_ICONS = {
  'schema/manufacturer': 'icons/manufacturer.png',
  'schema/accessory': 'icons/accessory.png',
  'schema/archive': 'icons/archive.png',
  'schema/battery': 'icons/battery.png',
  'schema/condition': 'icons/condition.png',
  'schema/exposureprogram': 'icons/exposureprogram.png',
  'schema/flashprotocol': 'icons/flashprotocol.png',
  'schema/filter': 'icons/filter.png',
  'schema/negativesize': 'icons/negativesize.png',
  'schema/format': 'icons/format.png',
  'schema/series': 'icons/series.png',
  'schema/flash': 'icons/flash.png',
  'schema/enlarger': 'icons/enlarger.png',
  'schema/meteringmode': 'icons/meteringmode.png',
  'schema/meteringtype': 'icons/meteringtype.png',
  'schema/mount': 'icons/mount.png',
  'schema/paperstock': 'icons/paperstock.png',
  'schema/person': 'icons/person.png',
  'schema/process': 'icons/process.png',
  'schema/teleconverter': 'icons/teleconverter.png',
  'schema/toner': 'icons/toner.png',
  'schema/filmstock': 'icons/filmstock.png',
  'schema/bulkfilm': 'icons/bulkfilm.png',
  'schema/shutterspeed': 'icons/shutterspeed.png',
  'schema/developer': 'icons/developer.png',
  'schema/lensmodel': 'icons/lensmodel.png',
  'schema/cameramodel': 'icons/cameramodel.png',
  'schema/lens': 'icons/lens.png',
  'schema/camera': 'icons/camera.png',
  'schema/film': 'icons/film.png',
  'schema/negative': 'icons/negative.png',
  'schema/print': 'icons/print.png',
  'schema/repair': 'icons/repair.png',
  'schema/scan': 'icons/scan.png',
  'schema/order': 'icons/order.png',
}

from django.utils.translation import ugettext_lazy as _
FLUENT_DASHBOARD_APP_GROUPS = (
    (_('Administration'), {
        'models': (
            'django.contrib.auth.*',
        ),
    }),
    (_('Quick links'), {
        'models': (
            'schema.models.Camera',
            'schema.models.Lens',
            'schema.models.Film',
            'schema.models.Negative',
            'schema.models.Print',
        ),
        'module': 'AppIconList',
        'collapsible': False,
    }),
    (_('Applications'), {
        'models': (
            'schema.*',
        ),
        'module': 'ModelList',
        'collapsible': False,
    }),
)

FAVICON_PATH = STATIC_URL + 'favicon.ico'


# Generated by Django 2.2.4 on 2019-10-15 09:26

from django.conf import settings
from django.db import migrations
import django.db.models.deletion
import django_currentuser.db.models.fields
import django_currentuser.middleware


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('schema', '0008_filmstocks'),
    ]

    operations = [
        migrations.AddField(
            model_name='accessory',
            name='owner',
            field=django_currentuser.db.models.fields.CurrentUserField(default=django_currentuser.middleware.get_current_authenticated_user, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='archive',
            name='owner',
            field=django_currentuser.db.models.fields.CurrentUserField(default=django_currentuser.middleware.get_current_authenticated_user, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='bulkfilm',
            name='owner',
            field=django_currentuser.db.models.fields.CurrentUserField(default=django_currentuser.middleware.get_current_authenticated_user, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='camera',
            name='owner',
            field=django_currentuser.db.models.fields.CurrentUserField(default=django_currentuser.middleware.get_current_authenticated_user, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='enlarger',
            name='owner',
            field=django_currentuser.db.models.fields.CurrentUserField(default=django_currentuser.middleware.get_current_authenticated_user, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='film',
            name='owner',
            field=django_currentuser.db.models.fields.CurrentUserField(default=django_currentuser.middleware.get_current_authenticated_user, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='filter',
            name='owner',
            field=django_currentuser.db.models.fields.CurrentUserField(default=django_currentuser.middleware.get_current_authenticated_user, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='filteradapter',
            name='owner',
            field=django_currentuser.db.models.fields.CurrentUserField(default=django_currentuser.middleware.get_current_authenticated_user, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='flash',
            name='owner',
            field=django_currentuser.db.models.fields.CurrentUserField(default=django_currentuser.middleware.get_current_authenticated_user, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='lens',
            name='owner',
            field=django_currentuser.db.models.fields.CurrentUserField(default=django_currentuser.middleware.get_current_authenticated_user, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='lightmeter',
            name='owner',
            field=django_currentuser.db.models.fields.CurrentUserField(default=django_currentuser.middleware.get_current_authenticated_user, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='mountadapter',
            name='owner',
            field=django_currentuser.db.models.fields.CurrentUserField(default=django_currentuser.middleware.get_current_authenticated_user, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='movie',
            name='owner',
            field=django_currentuser.db.models.fields.CurrentUserField(default=django_currentuser.middleware.get_current_authenticated_user, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='negative',
            name='owner',
            field=django_currentuser.db.models.fields.CurrentUserField(default=django_currentuser.middleware.get_current_authenticated_user, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='order',
            name='owner',
            field=django_currentuser.db.models.fields.CurrentUserField(default=django_currentuser.middleware.get_current_authenticated_user, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='person',
            name='owner',
            field=django_currentuser.db.models.fields.CurrentUserField(default=django_currentuser.middleware.get_current_authenticated_user, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='print',
            name='owner',
            field=django_currentuser.db.models.fields.CurrentUserField(default=django_currentuser.middleware.get_current_authenticated_user, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='projector',
            name='owner',
            field=django_currentuser.db.models.fields.CurrentUserField(default=django_currentuser.middleware.get_current_authenticated_user, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='repair',
            name='owner',
            field=django_currentuser.db.models.fields.CurrentUserField(default=django_currentuser.middleware.get_current_authenticated_user, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='scan',
            name='owner',
            field=django_currentuser.db.models.fields.CurrentUserField(default=django_currentuser.middleware.get_current_authenticated_user, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='series',
            name='owner',
            field=django_currentuser.db.models.fields.CurrentUserField(default=django_currentuser.middleware.get_current_authenticated_user, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='teleconverter',
            name='owner',
            field=django_currentuser.db.models.fields.CurrentUserField(default=django_currentuser.middleware.get_current_authenticated_user, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
    ]



from django.db import migrations

def forwards_func(apps, schema_editor):
  # We get the model from the versioned app registry;
  # if we directly import it, it'll be the wrong version
  db_alias = schema_editor.connection.alias

  Manufacturer = apps.get_model("schema", "Manufacturer")
  Manufacturer.objects.using(db_alias).bulk_create([
    Manufacturer(name="Boots", city="Nottingham", country="England", url="http://www.boots.com/", founded="1849"),
    Manufacturer(name="Braun", city="Nuremberg", country="Germany", url="http://www.braun-phototechnik.de/", founded="1915"),
    Manufacturer(name="Canon", city="Tokyo", country="Japan", url="http://www.canon.com/", founded="1937"),
    Manufacturer(name="Standard Cameras", country="England", founded="1931"),
    Manufacturer(name="Efke", city="Samobor", country="Croatia", founded="1947", dissolved="2012"),
    Manufacturer(name="Fuji", city="Tokyo", country="Japan", url="http://www.fujifilm.com/", founded="1934"),
    Manufacturer(name="Halina", country="China", founded="1906"),
    Manufacturer(name="Homemade", country="England"),
    Manufacturer(name="Ilford", city="Ilford", country="England", url="http://www.ilfordphoto.com/", founded="1879"),
    Manufacturer(name="KMZ", city="Krasnogorsk", country="Russia", url="http://www.zenit-foto.ru/", founded="1942"),
    Manufacturer(name="Kodak", city="Rochester", country="USA", url="http://www.kodak.com/", founded="1892"),
    Manufacturer(name="LOMO", city="St. Petersburg", country="Russia", url="http://www.lomoplc.com/", founded="1932", dissolved="1914"),
    Manufacturer(name="Maco", country="Germany", url="http://www.maco-photo.de/"),
    Manufacturer(name="Makinon", city="Tokyo", country="Japan", founded="1967", dissolved="1985"),
    Manufacturer(name="Mamiya", city="Tokyo", country="Japan", url="http://www.mamiyaleaf.com/", founded="1940"),
    Manufacturer(name="Olympus", city="Tokyo", country="Japan", url="http://www.olympus.co.uk/", founded="1919"),
    Manufacturer(name="Opteka", country="Japan", founded="2002"),
    Manufacturer(name="Samyang", city="Masan", country="South Korea", url="http://www.syopt.co.kr/", founded="1972"),
    Manufacturer(name="Tamron", city="Saitama", country="Japan", url="http://www.tamron.co.jp/en/", founded="1950"),
    Manufacturer(name="Unknown"),
    Manufacturer(name="Voigtlander", city="Vienna", country="Austria", url="http://www.voigtlaender.de/", founded="1756"),
    Manufacturer(name="Tokina", country="Japan", url="http://www.tokinalens.com/", founded="1970"),
    Manufacturer(name="Kenko", city="Tokyo", country="Japan", url="http://www.kenko-tokina.co.jp/e/index.html", founded="1928"),
    Manufacturer(name="Paragon", country="Japan"),
    Manufacturer(name="Agfa", city="Berlin", country="Germany", founded="1867"),
    Manufacturer(name="LPL", country="Japan", url="http://www.lpl-web.co.jp/", founded="1953"),
    Manufacturer(name="Durst", country="Italy", founded="1936"),
    Manufacturer(name="Toshikato", country="Japan"),
    Manufacturer(name="Vivitar", country="USA", url="http://www.vivitar.com/", founded="1938"),
    Manufacturer(name="Tudor", country="Japan"),
    Manufacturer(name="Rollei", city="Braunschweig", country="Germany", url="http://www.rollei.com/", founded="1920"),
    Manufacturer(name="Komamura", country="Japan", founded="1933"),
    Manufacturer(name="Ensign", city="London", country="England", founded="1903"),
    Manufacturer(name="Ross", city="London", country="England", founded="1830"),
    Manufacturer(name="Kentmere", city="Kentmere", country="England", url="http://www.kentmere.co.uk/"),
    Manufacturer(name="Jessops", city="Leicester", country="England", url="http://www.jessops.com/", founded="1935"),
    Manufacturer(name="Cosina", city="Nakano", country="Japan", url="http://www.cosina.co.jp/seihin/voigt/english/", founded="1959"),
    Manufacturer(name="Manfrotto", city="Cassola", country="Italy", url="http://www.manfrotto.co.uk/", founded="1974"),
    Manufacturer(name="Pentax", city="Tokyo", country="Japan", url="http://www.pentax.co.uk/", founded="1919"),
    Manufacturer(name="Blazzeo", country="China"),
    Manufacturer(name="Cobra"),
    Manufacturer(name="Sunpak", country="Japan", url="http://www.sunpak.jp/english/", founded="1963"),
    Manufacturer(name="Miranda", country="Japan", founded="1955"),
    Manufacturer(name="Hanimex", country="Australia", founded="1947"),
    Manufacturer(name="Ohnar", country="Japan"),
    Manufacturer(name="Celestron", country="USA", url="http://www.celestron.com/", founded="1964"),
    Manufacturer(name="Sigma", country="Japan", url="http://www.sigma-imaging-uk.com/", founded="1961"),
    Manufacturer(name="Lancaster", country="England", founded="1835"),
    Manufacturer(name="Schneider-Kreuznach", country="Germany", url="http://www.schneiderkreuznach.com/foto_e/foto", founded="1913"),
    Manufacturer(name="Leitz", country="Germany", url="http://uk.leica-camera.com/home/", founded="1913"),
    Manufacturer(name="Aldis", country="England", founded="1901"),
    Manufacturer(name="Revelation", country="Taiwan"),
    Manufacturer(name="Pentacon", city="Dresden", country="Germany", url="http://www.pentacon-dresden.de/", founded="1959"),
    Manufacturer(name="Chinon", city="Nagano", country="Japan", url="http://www.chinon.co.jp/", founded="1948", dissolved="2004"),
    Manufacturer(name="Dollond & Newcombe", country="England"),
    Manufacturer(name="Loonar Goupe", country="China"),
    Manufacturer(name="Polaroid", country="USA", url="http://www.polaroid.com/", founded="1937"),
    Manufacturer(name="The Impossible Project", city="Enschede", country="Netherlands", url="https://www.the-impossible-project.com/", founded="2008"),
    Manufacturer(name="Lomography", country="Austria", url="http://www.lomography.com/", founded="1992"),
    Manufacturer(name="Yongnuo", country="China", url="http://en.yongnuo.com.cn/"),
    Manufacturer(name="Graflex", country="USA", url="http://graflex.org/", founded="1898"),
    Manufacturer(name="Tokyo Kogaku", country="Japan", url="http://www.topcon.co.jp/en/index.html", founded="1932"),
    Manufacturer(name="Linhof", country="Germany", url="http://www.linhof.com/index-e.html", founded="1887"),
    Manufacturer(name="De Vere", country="England"),
    Manufacturer(name="Nikon", city="Tokyo", country="Japan", url="http://www.nikkor.com/", founded="1932"),
    Manufacturer(name="Konica", city="Tokyo", country="Japan", founded="1873", dissolved="2003"),
    Manufacturer(name="Feinwerk Technik", country="Germany"),
    Manufacturer(name="Meopta", city="Bratislava", country="Slovakia", founded="1933"),
    Manufacturer(name="ORWO", city="Wolfen", country="Germany", url="http://www.filmotec.de/"),
    Manufacturer(name="Fotospeed", city="Corsham", country="England", url="http://www.fotospeed.com/"),
    Manufacturer(name="Shackman", city="London", country="England"),
    Manufacturer(name="Dallmeyer", country="England", founded="1860"),
    Manufacturer(name="Sekonic"),
    Manufacturer(name="Zeiss Ikon", country="Germany"),
    Manufacturer(name="Realt", country="France"),
    Manufacturer(name="Soligor", city="Stuttgart", country="Germany", founded="1968"),
    Manufacturer(name="Bowens", city="London", country="England", founded="1923"),
    Manufacturer(name="Optomax", country="Japan"),
    Manufacturer(name="Philips", country="Holland", founded="1891"),
    Manufacturer(name="Bausch & Lomb"),
    Manufacturer(name="Bell & Howell"),
    Manufacturer(name="Kitvision", country="UK"),
    Manufacturer(name="MDT"),
    Manufacturer(name="Jacquard", country="USA", founded="1988"),
    Manufacturer(name="Neewer", country="China"),
    Manufacturer(name="Holga"),
    Manufacturer(name="Kawauso-Shoten", country="Japan"),
    Manufacturer(name="Minolta"),
    Manufacturer(name="Roeschlein-Kreuznach", country="Germany"),
    Manufacturer(name="Staeble", country="Germany", founded="1908"),
    Manufacturer(name="Tetenal"),
    Manufacturer(name="Foma"),
    Manufacturer(name="Tucht", city="Dusseldorf", country="Germany"),
    Manufacturer(name="Rodenstock", country="Germany"),
    Manufacturer(name="Roniflex"),
  ])


class Migration(migrations.Migration):

    dependencies = [
        ('schema', '0003_shutter_speeds')
    ]

    operations = [
        migrations.RunPython(forwards_func),
    ]
